# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_FileSystemCapabilities class.

LMI_FileSystemCapabilities
---------------------------------------

.. autoclass:: LMI_FileSystemCapabilities
    :members:

"""

from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_FileSystemCapabilities(CapabilitiesProvider):
    """
        LMI_FileSystemCapabilities provider implementation.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_FileSystemCapabilities, self).__init__(
                "LMI_FileSystemCapabilities", *args, **kwargs)

        vals = self.Values
        self.instances = [
            {
                'InstanceID': 'LMI:LMI_FileSystemCapabilities:ext2',
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT2,
                'SupportedPersistenceTypes':
                    [ vals.SupportedPersistenceTypes.Persistent ],
                'SupportedProperties':
                    [
                            vals.SupportedProperties.DataExtentsSharing,
                            vals.SupportedProperties.FilenameCaseAttributes,
                            vals.SupportedProperties.FilenameFormats,
                            vals.SupportedProperties.Persistence,
                    ],
            },
            {
                'InstanceID': 'LMI:LMI_FileSystemCapabilities:ext3',
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT3,
                'SupportedPersistenceTypes':
                    [ vals.SupportedPersistenceTypes.Persistent ],
                'SupportedProperties':
                    [
                            vals.SupportedProperties.DataExtentsSharing,
                            vals.SupportedProperties.FilenameCaseAttributes,
                            vals.SupportedProperties.FilenameFormats,
                            vals.SupportedProperties.Persistence,
                    ],
            },
            {
                'InstanceID': 'LMI:LMI_FileSystemCapabilities:ext4',
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT4,
                'SupportedPersistenceTypes':
                    [ vals.SupportedPersistenceTypes.Persistent ],
                'SupportedProperties':
                    [
                            vals.SupportedProperties.DataExtentsSharing,
                            vals.SupportedProperties.FilenameCaseAttributes,
                            vals.SupportedProperties.FilenameFormats,
                            vals.SupportedProperties.Persistence,
                    ],
            },
            {
                'InstanceID': 'LMI:LMI_FileSystemCapabilities:btrfs',
                'ActualFileSystemType':
                    vals.ActualFileSystemType.BTRFS,
                'SupportedPersistenceTypes':
                    [ vals.SupportedPersistenceTypes.Persistent ],
                'SupportedProperties':
                    [
                            vals.SupportedProperties.DataExtentsSharing,
                            vals.SupportedProperties.FilenameCaseAttributes,
                            vals.SupportedProperties.FilenameFormats,
                            vals.SupportedProperties.Persistence,
                    ],
            },
            {
                'InstanceID': 'LMI:LMI_FileSystemCapabilities:xfs',
                'ActualFileSystemType':
                    vals.ActualFileSystemType.XFS,
                'SupportedPersistenceTypes':
                    [ vals.SupportedPersistenceTypes.Persistent ],
                'SupportedProperties':
                    [
                            vals.SupportedProperties.DataExtentsSharing,
                            vals.SupportedProperties.FilenameCaseAttributes,
                            vals.SupportedProperties.FilenameFormats,
                            vals.SupportedProperties.Persistence,
                    ],
            },
        ]


    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """
        return self.instances

    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "Application-specific FileSystemSetting is not yet supported.")

    class Values(CapabilitiesProvider.Values):
        class ActualFileSystemType(object):
            Unknown = pywbem.Uint16(0)
            UFS = pywbem.Uint16(2)
            HFS = pywbem.Uint16(3)
            FAT = pywbem.Uint16(4)
            FAT16 = pywbem.Uint16(5)
            FAT32 = pywbem.Uint16(6)
            NTFS4 = pywbem.Uint16(7)
            NTFS5 = pywbem.Uint16(8)
            XFS = pywbem.Uint16(9)
            AFS = pywbem.Uint16(10)
            EXT2 = pywbem.Uint16(11)
            EXT3 = pywbem.Uint16(12)
            REISERFS = pywbem.Uint16(13)
            # DMTF_Reserved = ..
            EXT4 = pywbem.Uint16(32769)
            BTRFS = pywbem.Uint16(32770)
            JFS = pywbem.Uint16(32771)
            TMPFS = pywbem.Uint16(32772)
            VFAT = pywbem.Uint16(32773)

        class SupportedPersistenceTypes(object):
            Other = pywbem.Uint16(1)
            Persistent = pywbem.Uint16(2)
            Temporary = pywbem.Uint16(3)
            External = pywbem.Uint16(4)

        class SupportedProperties(object):
            DataExtentsSharing = pywbem.Uint16(2)
            CopyTarget = pywbem.Uint16(3)
            FilenameCaseAttributes = pywbem.Uint16(4)
            FilenameStreamFormats = pywbem.Uint16(5)
            FilenameFormats = pywbem.Uint16(6)
            LockingSemantics = pywbem.Uint16(7)
            AuthorizationProtocols = pywbem.Uint16(8)
            AuthenticationProtocols = pywbem.Uint16(9)
            Persistence = pywbem.Uint16(10)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for ServiceProvider class.

ServiceProvider
---------------

.. autoclass:: ServiceProvider
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.ComputerSystem import get_system_name

class ServiceProvider(BaseProvider):
    """
        Base class for every LMI_*Service providers.
        It implements get_instance and enum_instances methods.
    """

    @cmpi_logging.trace_method
    def __init__(self, classname, *args, **kwargs):
        super(ServiceProvider, self).__init__(*args, **kwargs)
        self.classname = classname


    @cmpi_logging.trace_method
    def check_instance(self, model):
        """
            Check if the model represents real instance of this class.
            Throw an error if not.
        """
        if (model['SystemCreationClassName'] != self.config.system_class_name
                or model['SystemName'] != get_system_name()
                or model['CreationClassName'] != self.classname
                or model['Name'] != self.classname):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, "Wrong keys.")

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        self.check_instance(model)

        model['EnabledDefault'] = self.Values.EnabledDefault.Enabled
        model['EnabledState'] = self.Values.EnabledState.Enabled
        model['HealthState'] = self.Values.HealthState.OK
        model['OperationalStatus'] = [self.Values.OperationalStatus.OK, ]
        model['PrimaryStatus'] = self.Values.PrimaryStatus.OK
        model['Started'] = True
        model['StartMode'] = self.Values.StartMode.Automatic
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'CreationClassName': None, 'SystemName': None,
            'Name': None, 'SystemCreationClassName': None})
        model['SystemName'] = get_system_name()
        model['SystemCreationClassName'] = self.config.system_class_name
        model['CreationClassName'] = self.classname
        model['Name'] = self.classname
        if keys_only:
            yield model
        else:
            yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    def _get_instance_name(self):
        """ Return CIMInstanceName of the service singleton."""
        name = pywbem.CIMInstanceName(
                classname=self.classname,
                namespace=self.config.namespace,
                keybindings={
                    'SystemName': get_system_name(),
                    'SystemCreationClassName': self.config.system_class_name,
                    'CreationClassName': self.classname,
                    'Name': self.classname})
        return name

    class Values(object):
        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            Relocating = pywbem.Uint16(19)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class PrimaryStatus(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(1)
            Degraded = pywbem.Uint16(2)
            Error = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class StartMode(object):
            Automatic = 'Automatic'
            Manual = 'Manual'



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_DiskPartitionConfigurationSetting class.

LMI_DiskPartitionConfigurationSetting
-------------------------------------

.. autoclass:: LMI_DiskPartitionConfigurationSetting
    :members:

"""

import pywbem
from lmi.storage.SettingProvider import SettingProvider
from lmi.storage.SettingManager import Setting

import parted
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.util import storage

class LMI_DiskPartitionConfigurationSetting(SettingProvider):
    """
        Implementation of LMI_DiskPartitionConfigurationSetting
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        supported_properties = {
            'Bootable': SettingProvider.string_to_bool,
            'Hidden': SettingProvider.string_to_bool,
            'PartitionType': pywbem.Uint16,
        }
        super(LMI_DiskPartitionConfigurationSetting, self).__init__(
                setting_classname="LMI_DiskPartitionConfigurationSetting",
                supported_properties=supported_properties,
                validate_properties=None,
                *args, **kwargs)

    @cmpi_logging.trace_method
    def get_configuration(self, device):
        """
            Return Setting with configuration of given device.
        """
        _id = self.create_setting_id(storage.get_persistent_name(device))
        setting = self.setting_manager.create_setting(
                self.setting_classname,
                Setting.TYPE_CONFIGURATION,
                _id)
        setting['Bootable'] = str(device.bootable)
        flag = device.getFlag(parted.PARTITION_HIDDEN)
        if flag:
            setting['Hidden'] = "True"
        if device.isExtended:
            setting['PartitionType'] = str(self.Values.PartitionType.Extended)
        elif device.isLogical:
            setting['PartitionType'] = str(self.Values.PartitionType.Logical)
        elif device.isPrimary:
            setting['PartitionType'] = str(self.Values.PartitionType.Primary)
        else:
            setting['PartitionType'] = str(self.Values.PartitionType.Unknown)
        setting['ElementName'] = setting.the_id
        return setting

    @cmpi_logging.trace_method
    def enumerate_configurations(self):
        """
            Enumerate all instances attached to partitions.
        """
        for device in self.storage.partitions:
            yield self.get_configuration(device)

    @cmpi_logging.trace_method
    def get_configuration_for_id(self, instance_id):
        """
            Return Setting instance for given instance_id.
            Return None if no such Setting is found.
        """
        path = self.parse_setting_id(instance_id)
        if not path:
            return None

        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        return self.get_configuration(device)

    @cmpi_logging.trace_method
    def get_associated_element_name(self, instance_id):
        """
            Return CIMInstanceName for ElementSettingData association.
            Return None if no such element exist.
        """
        path = self.parse_setting_id(instance_id)
        if not path:
            return None

        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None

        return self.provider_manager.get_name_for_device(device)


    class Values(SettingProvider.Values):
        class PartitionType(object):
            Unknown = 0
            Primary = 1
            Extended = 2
            Logical = 3



# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

"""
    This is a simple wrapper to gather test coverage of OpenLMI storage
    providers.

    It is special provider module, which starts collecting the coverage
    and just calls cimom_entry provider module functions.
"""

import os

COVERAGE_DIRECTORY = "/var/lib/openlmi-storage/coverage/"
LOGFILE_NAME = COVERAGE_DIRECTORY + "coverage.err"
COVERAGE_FILE = COVERAGE_DIRECTORY + "coverage."

def start_coverage():
    """ Start collecting coverage. """
    # create directory for logs
    if not os.path.isdir(COVERAGE_DIRECTORY):
        os.makedirs(COVERAGE_DIRECTORY)

    # find unused file name
    i = 0
    while os.path.exists(COVERAGE_FILE + str(i)):
        i = i + 1
    coverage_file = COVERAGE_FILE + str(i)

    try:
        print "Loading coverage"
        import coverage
        cov = coverage.coverage(auto_data=True,
                data_file=coverage_file,
                timid=True)
        # remove any previous error file
        if os.path.exists(LOGFILE_NAME):
            os.remove(LOGFILE_NAME)
        print "Starting coverage"
        cov.start()
    except ImportError:
        msg = "'import coverage' failed, no coverage report will be generated."
        print(msg)
        log = open(LOGFILE_NAME, "w")
        log.write(msg + "\n")
        log.close()
        cov = None
    return cov

# Pylint expects that only constants are on global level, but we have one
# global variable + we assign also functions below.
# Also we *do* check if the wrapped provider module has all methods
# we access (with hasattr(cimom_entry,...))
# pylint: disable-msg=C0103,E1101

mycoverage = start_coverage()
# import the cimom_entry only when the coverage has started
# (so we can measure the initialization coverage)
import lmi.storage.cimom_entry as cimom_entry

def shutdown(env):
    """Called from CIMOM on shutdown."""
    if mycoverage:
        print "Stopping coverage"
        mycoverage.stop()
        mycoverage.save()
    if hasattr(cimom_entry, 'shutdown'):
        return cimom_entry.shutdown(env)

# forward all other provider module methods to cmpi_bindings
if hasattr(cimom_entry, 'init'):
    init = cimom_entry.init
if hasattr(cimom_entry, 'get_providers'):
    get_providers = cimom_entry.get_providers
if hasattr(cimom_entry, 'can_unload'):
    can_unload = cimom_entry.can_unload
if hasattr(cimom_entry, 'handle_indication'):
    handle_indication = cimom_entry.handle_indication
if hasattr(cimom_entry, 'authorize_filter'):
    authorize_filter = cimom_entry.authorize_filter
if hasattr(cimom_entry, 'activate_filter'):
    activate_filter = cimom_entry.activate_filter
if hasattr(cimom_entry, 'deactivate_filter'):
    deactivate_filter = cimom_entry.deactivate_filter
if hasattr(cimom_entry, 'enable_indications'):
    enable_indications = cimom_entry.enable_indications
if hasattr(cimom_entry, 'disable_indications'):
    disable_indications = cimom_entry.disable_indications

# pylint: enable-msg=C0103,E1101



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
"""
Python Provider for LMI_ExtentEncryptionConfigurationService

Instruments the CIM class LMI_ExtentEncryptionConfigurationService
"""

import pywbem
import blivet.devicelibs.crypto as crypto
import blivet.formats.luks
import blivet.devices
from blivet.errors import CryptoError
from lmi.storage.ServiceProvider import ServiceProvider
from lmi.storage.util.storage import *
from lmi.providers.JobManager import Job
import lmi.storage.util
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_ExtentEncryptionConfigurationService(ServiceProvider):
    """Instrument the CIM class LMI_ExtentEncryptionConfigurationService

    A Service is a LogicalElement that represents the availability of
    functionality that can be managed. This functionality may be provided
    by a seperately modeled entity such as a LogicalDevice or a
    SoftwareFeature, or both. The modeled Service typically provides only
    functionality required for management of itself or the elements it
    affects.

    """

    @cmpi_logging.trace_method
    def __init__ (self, *args, **kwargs):
        super(LMI_ExtentEncryptionConfigurationService, self).__init__(
            classname="LMI_ExtentEncryptionConfigurationService",
            *args, **kwargs)

    @cmpi_logging.trace_method
    def cim_method_createencryptionformat(self, env, object_name,
                                          param_goal=None,
                                          param_inextent=None,
                                          param_passphrase=None):
        """Implements LMI_ExtentEncryptionConfigurationService.CreateEncryptionFormat()

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CreateEncryptionFormat()
            should be invoked.
        param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormatSetting', ...))
        param_inextent --  The input parameter InExtent (type REF (pywbem.CIMInstanceName(classname='CIM_StorageExtent', ...))
        param_passphrase --  The input parameter Passphrase (type unicode)

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
        Format -- (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormat', ...))

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        args = {'InExtent':param_inextent,
                'Passphrase':param_passphrase}
        check_empty_parameters(**args)

        device = self.provider_manager.get_device_for_name(param_inextent)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Can't find any device for %s." % param_inextent['DeviceID'])

        # remember input parameters for job
        input_arguments = {
            'Goal' : pywbem.CIMProperty(name='Goal',
                                        type='reference',
                                        value=param_goal),
            'InExtent' : pywbem.CIMProperty(name='InExtent',
                                            type='reference',
                                            value=param_inextent),
            'Passphrase' : pywbem.CIMProperty(name='Passphrase',
                                              type='string',
                                              value=param_passphrase)}

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="LUKS FORMAT %s" % device.path,
                  input_arguments=input_arguments,
                  method_name="CreateEncryptionFormat",
                  affected_elements=[param_inextent],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._create_encryption_format, job,
                               param_inextent, param_passphrase)
        self.job_manager.add_job(job)

        rval = self.EncryptionFormatMethods.Method_Parameters_Checked___Job_Started
        outparams = [(pywbem.CIMParameter(
                name='Job',
                type='reference',
                value=job.get_name()))]
        return (rval, outparams)

    @cmpi_logging.trace_method
    def _create_encryption_format(self, job, devicename, passphrase):
        """
        Really create encryption format. All checks have been done.
        """
        device = self.provider_manager.get_device_for_name(devicename)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Device disappeared: " + devicename['DeviceID'])

        rval = self.EncryptionFormatMethods.Job_Completed_with_No_Error
        state = Job.STATE_FINISHED_OK
        err = None

        # destroy previous format
        destroyAction = blivet.ActionDestroyFormat(device)
        fmt = blivet.formats.luks.LUKS(device=device.path, passphrase=passphrase, mapName=None)
        action = blivet.ActionCreateFormat(device, format=fmt)
        lmi.storage.util.storage.do_storage_action(self.storage, [destroyAction, action])

        new_device = self.storage.devicetree.getDeviceByPath(device.path)
        if not new_device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot locate new format, was it removed?")
        # XXX after blivet.reset(), the device gets opened by blivet. see BUG #996118

        efname = self.provider_manager.get_name_for_format(new_device, new_device.format)
        outparams = {'Format' : efname}
        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            output_arguments=outparams,
            affected_elements=[efname],
            error=err)

    @cmpi_logging.trace_method
    def cim_method_openencryptionformat(self, env, object_name,
                                        param_elementname=None,
                                        param_passphrase=None,
                                        param_format=None):
        """Implements LMI_ExtentEncryptionConfigurationService.OpenEncryptionFormat()

        Opens a LUKS device. This usually means an entry accessible through
        the device mapper.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method OpenEncryptionFormat()
            should be invoked.
        param_elementname --  The input parameter ElementName (type unicode)
        param_passphrase --  The input parameter Passphrase (type unicode)
        param_format --  The input parameter Format (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormat', ...))

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.OpenEncryptionFormat)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
        Extent -- (type REF (pywbem.CIMInstanceName(classname='CIM_StorageExtent', ...))

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        args = {'Format':param_format,
                'Passphrase':param_passphrase,
                'ElementName':param_elementname}
        check_empty_parameters(**args)

        provider = self.provider_manager.get_provider_for_format_name(param_format)
        (device, _) = provider.get_format_for_name(param_format)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Can't find any device for %s." % param_format['Name'])

        # remember input parameters for job
        input_arguments = {
            'ElementName' : pywbem.CIMProperty(name='ElementName',
                                               type='string',
                                               value=param_elementname),
            'Format' : pywbem.CIMProperty(name='Format',
                                            type='reference',
                                            value=param_format),
            'Passphrase' : pywbem.CIMProperty(name='Passphrase',
                                              type='string',
                                              value=param_passphrase)}

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="LUKS OPEN %s" % device.path,
                  input_arguments=input_arguments,
                  method_name="OpenEncryptionFormat",
                  affected_elements=[param_format],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._open_encryption_format, job,
                               param_format, param_elementname, param_passphrase)
        self.job_manager.add_job(job)

        rval = self.EncryptionFormatMethods.Method_Parameters_Checked___Job_Started
        outparams = [(pywbem.CIMParameter(
                name='Job',
                type='reference',
                value=job.get_name()))]
        return (rval, outparams)


    @cmpi_logging.trace_method
    def _open_encryption_format(self, job, formatname, elementname, passphrase):
        """
        Really open encryption format. All checks have been done.
        """
        provider = self.provider_manager.get_provider_for_format_name(formatname)
        (device, fmt) = provider.get_format_for_name(formatname)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Device disappeared: " + formatname['Name'])
        if not fmt or not isinstance(fmt, blivet.formats.luks.LUKS):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "LUKS format disappeared from device: " + formatname['Name'])

        children = self.storage.devicetree.getChildren(device)
        if children:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "LUKS is already opened as %s" % (children[0]))

        rval = self.EncryptionFormatMethods.Job_Completed_with_No_Error
        state = Job.STATE_FINISHED_OK
        err = None

        device.format.mapName = elementname
        device.format.passphrase = passphrase
        luksdevice = blivet.devices.LUKSDevice(elementname,
                                               size=device.size,
                                               uuid=device.format.uuid,
                                               sysfsPath=device.sysfsPath,
                                               format=device.format,
                                               parents=[device],
                                               exists=False)
        # See BUG #996457.
        luksdevice.format.device = device.path
        action = blivet.ActionCreateDevice(luksdevice)
        lmi.storage.util.storage.do_storage_action(self.storage, [action])

        new_device = self.storage.devicetree.getDeviceByPath(luksdevice.path)
        if not new_device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot locate new format, was it removed?")

        extent = self.provider_manager.get_name_for_device(new_device)
        outparams = {'Extent' : extent}
        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            output_arguments=outparams,
            affected_elements=[extent],
            error=err)

    @cmpi_logging.trace_method
    def cim_method_closeencryptionformat(self, env, object_name,
                                         param_format=None):
        """Implements LMI_ExtentEncryptionConfigurationService.CloseEncryptionFormat()

        Closes a LUKS device.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CloseEncryptionFormat()
            should be invoked.
        param_format --  The input parameter Format (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormat', ...))

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.CloseEncryptionFormat)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        args = {'Format':param_format}
        check_empty_parameters(**args)

        provider = self.provider_manager.get_provider_for_format_name(param_format)
        (device, _) = provider.get_format_for_name(param_format)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Can't find any device for %s." % param_format['Name'])

        # remember input parameters for job
        input_arguments = {'Format' : pywbem.CIMProperty(name='Format',
                                                         type='reference',
                                                         value=param_format)}

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="LUKS CLOSE %s" % device.path,
                  input_arguments=input_arguments,
                  method_name="CloseEncryptionFormat",
                  affected_elements=[param_format],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._close_encryption_format, job, param_format)
        self.job_manager.add_job(job)

        rval = self.EncryptionFormatMethods.Method_Parameters_Checked___Job_Started
        outparams = [(pywbem.CIMParameter(
                name='Job',
                type='reference',
                value=job.get_name()))]
        return (rval, outparams)

    @cmpi_logging.trace_method
    def _close_encryption_format(self, job, formatname):
        """
        Really close encryption format. All checks have been done.
        """
        provider = self.provider_manager.get_provider_for_format_name(formatname)
        (device, fmt) = provider.get_format_for_name(formatname)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Device disappeared: " + formatname['Name'])
        if not fmt or not isinstance(fmt, blivet.formats.luks.LUKS):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "LUKS format disappeared from device: " + formatname['Name'])
        children = self.storage.devicetree.getChildren(device)
        if not children:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "No LUKS device opened: " + formatname['Name'])

        rval = self.EncryptionFormatMethods.Job_Completed_with_No_Error
        state = Job.STATE_FINISHED_OK
        err = None

        luksdevice = children[0]
        action = blivet.ActionDestroyDevice(luksdevice)
        lmi.storage.util.storage.do_storage_action(self.storage, [action])

        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            output_arguments={},
            affected_elements=[],
            error=err)

    @cmpi_logging.trace_method
    def cim_method_addpassphrase(self, env, object_name,
                                 param_newpassphrase=None,
                                 param_passphrase=None,
                                 param_format=None):
        """Implements LMI_ExtentEncryptionConfigurationService.AddPassphrase()

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method AddPassphrase()
            should be invoked.
        param_newpassphrase --  The input parameter NewPassphrase (type unicode)
        param_passphrase --  The input parameter Passphrase (type unicode)
        param_format --  The input parameter Format (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormat', ...))
        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        args = {'Format':param_format,
                'Passphrase':param_passphrase,
                'NewPassphrase':param_newpassphrase}
        check_empty_parameters(**args)

        provider = self.provider_manager.get_provider_for_format_name(param_format)
        if not provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find given Format.")

        (device, fmt) = provider.get_format_for_name(param_format)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find given Format.")

        try:
            crypto.luks_add_key(device.path, param_newpassphrase, param_passphrase)
        except CryptoError:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Can't add key to LUKS.")

        return (pywbem.Uint32(0), [])

    @cmpi_logging.trace_method
    def cim_method_deletepassphrase(self, env, object_name,
                                    param_passphrase=None,
                                    param_format=None):
        """Implements LMI_ExtentEncryptionConfigurationService.DeletePassphrase()

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method DeletePassphrase()
            should be invoked.
        param_passphrase --  The input parameter Passphrase (type unicode)
        param_format --  The input parameter Format (type REF (pywbem.CIMInstanceName(classname='LMI_EncryptionFormat', ...))

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        args = {'Format':param_format,
                'Passphrase':param_passphrase}
        check_empty_parameters(**args)

        provider = self.provider_manager.get_provider_for_format_name(param_format)
        if not provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find given Format.")

        (device, fmt) = provider.get_format_for_name(param_format)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find given Format.")

        try:
            crypto.luks_remove_key(device.path, None, param_passphrase)
        except CryptoError:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Can't remove key from LUKS.")

        return (pywbem.Uint32(0), [])

    class EncryptionFormatMethods(object):
        Job_Completed_with_No_Error = pywbem.Uint32(0)
        Not_Supported = pywbem.Uint32(1)
        Unknown = pywbem.Uint32(2)
        Timeout = pywbem.Uint32(3)
        Failed = pywbem.Uint32(4)
        Invalid_Parameter = pywbem.Uint32(5)
        In_Use = pywbem.Uint32(6)
        # DMTF_Reserved = ..
        Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
        # Method_Reserved = 4098..32767
        # Vendor_Specific = 32768..65535



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase, short_tests_only
import unittest
import pywbem


MEGABYTE = 1024 * 1024

class TestCreatePool(StorageTestBase):
    """
        Test LMI_StorageConfigurationService.CreateOrModifyStoragePool
        (create only).
    """

    DISK_CLASS = "LMI_StorageExtent"
    VG_CLASS = "LMI_VGStoragePool"
    STYLE_EMBR = 4100
    STYLE_MBR = 2
    STYLE_GPT = 3
    PARTITION_CLASS = "LMI_GenericDiskPartition"


    def setUp(self):
        """ Find storage service. """
        super(TestCreatePool, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        self.part_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]
        self.capabilities = self.wbemconnection.EnumerateInstanceNames(
                "LMI_VGStorageCapabilities")[0]

    def _get_disk_size(self, diskname):
        """ Return size of given disk, in bytes."""
        disk = self.wbemconnection.GetInstance(diskname)
        return disk['NumberOfBlocks'] * disk['BlockSize']

    def test_create_1pv(self):
        """ Test CreateOrModifyStoragePool with one PV."""
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyStoragePool",
                self.service,
                int, "pool",
                InExtents=self.partition_names[:1],
                ElementName='tstName')
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('pool'):
                vg = self.wbemconnection.GetInstance(outparams['pool'])
                outparams['size'] = vg['TotalExtents'] * vg['ExtentSize']

        self.assertEqual(ret, 0)
        self.assertEqual(len(outparams), 2)
        self.assertAlmostEqual(
                outparams['size'],
                self._get_disk_size(self.partition_names[0]),
                delta=4 * MEGABYTE)
        vgname = outparams['pool']
        vg = self.wbemconnection.GetInstance(vgname)
        self.assertEqual(vg['TotalManagedSpace'], outparams['size'])
        self.assertEqual(vg['PoolID'], 'tstName')
        self.assertEqual(vg['ElementName'], 'tstName')
        self.assertNotEqual(vg['UUID'], '')
        self.assertNotEqual(vg['UUID'], None)
        self.assertEqual(vg['ExtentSize'], 4 * MEGABYTE)
        self.assertEqual(
                vg['ExtentSize'] * vg['TotalExtents'],
                vg['TotalManagedSpace'])
        self.assertEqual(
                vg['ExtentSize'] * vg['RemainingExtents'],
                vg['RemainingManagedSpace'])

        (ret, outparams) = self.invoke_async_method(
                'DeleteStoragePool',
                self.service,
                int, None,
                Pool=vgname)
        self.assertEquals(ret, 0)

    @unittest.skipIf(short_tests_only(), "Running short tests only.")
    def test_create_10pv(self):
        """ Test CreateOrModifyStoragePool with 10 PVs."""
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyStoragePool",
                self.service,
                int, "pool",
                InExtents=self.partition_names[:10])

        vg = self.wbemconnection.GetInstance(outparams['pool'])

        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            outparams['size'] = vg['TotalExtents'] * vg['ExtentSize']

        self.assertEqual(ret, 0)
        self.assertEqual(len(outparams), 2)
        sizes = map(lambda p: self._get_disk_size(p), self.partition_names[:10])
        pv_unusable = sum(map(lambda s: s % vg['ExtentSize'], sizes))
        self.assertEqual(
                outparams['size'],
                sum(sizes) - pv_unusable)
        vg = outparams['pool']

        (ret, outparams) = self.invoke_async_method(
                'DeleteStoragePool',
                self.service,
                int, None,
                Pool=vg)
        self.assertEquals(ret, 0)

    @unittest.skipIf(short_tests_only(), "Running short tests only.")
    def test_create_10vg(self):
        """ Test CreateOrModifyStoragePool with 10 VGs."""
        vgs = []
        for part in self.partition_names[:10]:
            (ret, outparams) = self.invoke_async_method(
                    "CreateOrModifyStoragePool",
                    self.service,
                    int, "pool",
                    InExtents=[part])
            if len(outparams) == 1:
                # there is no Size returned, Pegasus does not support it yet
                # TODO: remove when pegasus supports embedded objects of unknown
                # types, rhbz#920763
                if outparams.has_key('pool'):
                    vg = self.wbemconnection.GetInstance(outparams['pool'])
                    outparams['size'] = vg['TotalExtents'] * vg['ExtentSize']

            self.assertEqual(ret, 0)
            self.assertEqual(len(outparams), 2)
            vg = outparams['pool']
            vgs.append(vg)

        for vg in vgs:
            (ret, outparams) = self.invoke_async_method(
                    'DeleteStoragePool',
                    self.service,
                    int, None,
                    Pool=vg)
            self.assertEquals(ret, 0)

    def test_create_unknown_setting(self):
        """ Test CreateOrModifyStoragePool with non-existing setting."""
        goal = pywbem.CIMInstanceName(
                classname=" LMI_VGStorageSetting",
                keybindings={
                        'InstanceID' : 'LMI:LMI_VGStorageSetting:not-existing'
                })
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                    "CreateOrModifyStoragePool",
                    self.service,
                    InExtents=self.partition_names[:1],
                    Goal=goal
                    )

    def test_create_wrong_setting_class(self):
        """ Test CreateOrModifyStoragePool with non-existing setting."""
        goal = pywbem.CIMInstanceName(
                classname=" LMI_LVStorageSetting",
                keybindings={
                        'InstanceID' : 'LMI:LMI_LVStorageSetting:not-existing'
                })
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                    "CreateOrModifyStoragePool",
                    self.service,
                    InExtents=self.partition_names[:1],
                    Goal=goal
                    )

    def test_create_wrong_inpools(self):
        """ Test CreateOrModifyStoragePool with InPools param."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                    "CreateOrModifyStoragePool",
                    self.service,
                    InExtents=self.partition_names[:1],
                    InPools=self.partition_names[:1]
                    )

    def test_create_wrong_size(self):
        """ Test CreateOrModifyStoragePool with Size param."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                    "CreateOrModifyStoragePool",
                    self.service,
                    InExtents=self.partition_names[:1],
                    Size=pywbem.Uint64(1 * MEGABYTE)
                    )

    def _create_setting(self):
        """ Create a VGStorageSetting and return CIMInstance of it."""
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "CreateSetting",
                self.capabilities)
        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 1)

        setting_name = outparams['newsetting']
        setting = self.wbemconnection.GetInstance(setting_name)
        return setting

    def _delete_setting(self, setting_name):
        """ Delete given setting. """
        self.wbemconnection.DeleteInstance(setting_name)

    def test_create_default_setting(self):
        """
            Test CreateOrModifyStoragePool with default setting from 
            VGStroageCapabilities.CreateSetting.
        """
        goal = self._create_setting()

        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyStoragePool",
                self.service,
                int, "pool",
                InExtents=self.partition_names[:1],
                Goal=goal.path)
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('pool'):
                vg = self.wbemconnection.GetInstance(outparams['pool'])
                outparams['size'] = vg['TotalExtents'] * vg['ExtentSize']

        self.assertEqual(ret, 0)
        self.assertEqual(len(outparams), 2)
        self.assertAlmostEqual(
                outparams['size'],
                self._get_disk_size(self.partition_names[0]),
                delta=4 * MEGABYTE)
        vgname = outparams['pool']
        vg = self.wbemconnection.GetInstance(vgname)
        self.assertEqual(vg['TotalManagedSpace'], outparams['size'])
        self.assertNotEqual(vg['ElementName'], '')
        self.assertNotEqual(vg['ElementName'], None)
        self.assertNotEqual(vg['UUID'], '')
        self.assertNotEqual(vg['UUID'], None)
        self.assertEqual(vg['ExtentSize'], 4 * MEGABYTE)
        self.assertEqual(
                vg['ExtentSize'] * vg['TotalExtents'],
                vg['TotalManagedSpace'])
        self.assertEqual(
                vg['ExtentSize'] * vg['RemainingExtents'],
                vg['RemainingManagedSpace'])

        # check it has a setting associated
        settings = self.wbemconnection.Associators(
                vgname,
                AssocClass="LMI_VGElementSettingData")
        self.assertEqual(len(settings), 1)
        setting = settings[0]
        self.assertEqual(setting['ExtentSize'], goal['ExtentSize'])
        self.assertEqual(setting['DataRedundancyGoal'], goal['DataRedundancyGoal'])
        self.assertLessEqual(setting['DataRedundancyMax'], goal['DataRedundancyMax'])
        self.assertGreaterEqual(setting['DataRedundancyMin'], goal['DataRedundancyMin'])
        self.assertEqual(setting['ExtentStripeLength'], goal['ExtentStripeLength'])
        self.assertLessEqual(setting['ExtentStripeLengthMax'], goal['ExtentStripeLengthMax'])
        self.assertGreaterEqual(setting['ExtentStripeLengthMin'], goal['ExtentStripeLengthMin'])
        self.assertEqual(setting['NoSinglePointOfFailure'], goal['NoSinglePointOfFailure'])
        self.assertEqual(setting['PackageRedundancyGoal'], goal['PackageRedundancyGoal'])
        self.assertLessEqual(setting['PackageRedundancyMax'], goal['PackageRedundancyMax'])
        self.assertGreaterEqual(setting['PackageRedundancyMin'], goal['PackageRedundancyMin'])

        (ret, outparams) = self.invoke_async_method(
                'DeleteStoragePool',
                self.service,
                int, None,
                Pool=vgname)
        self.assertEquals(ret, 0)
        self._delete_setting(goal.path)

    def test_create_setting_1m(self):
        """
            Test CreateOrModifyStoragePool with 2MiB ExtentSize.
        """
        goal = self._create_setting()
        goal['ExtentSize'] = pywbem.Uint64(MEGABYTE)
        self.wbemconnection.ModifyInstance(goal)

        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyStoragePool",
                self.service,
                int, "pool",
                InExtents=self.partition_names[:1],
                Goal=goal.path)
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('pool'):
                vg = self.wbemconnection.GetInstance(outparams['pool'])
                outparams['size'] = vg['TotalExtents'] * vg['ExtentSize']

        self.assertEqual(ret, 0)
        self.assertEqual(len(outparams), 2)
        self.assertAlmostEqual(
                outparams['size'],
                self._get_disk_size(self.partition_names[0]),
                delta=4 * MEGABYTE)
        vgname = outparams['pool']
        vg = self.wbemconnection.GetInstance(vgname)
        self.assertEqual(vg['TotalManagedSpace'], outparams['size'])
        self.assertNotEqual(vg['ElementName'], '')
        self.assertNotEqual(vg['ElementName'], None)
        self.assertNotEqual(vg['UUID'], '')
        self.assertNotEqual(vg['UUID'], None)
        self.assertEqual(vg['ExtentSize'], MEGABYTE)
        self.assertEqual(
                vg['ExtentSize'] * vg['TotalExtents'],
                vg['TotalManagedSpace'])
        self.assertEqual(
                vg['ExtentSize'] * vg['RemainingExtents'],
                vg['RemainingManagedSpace'])

        # check it has a setting associated
        settings = self.wbemconnection.Associators(
                vgname,
                AssocClass="LMI_VGElementSettingData")
        self.assertEqual(len(settings), 1)
        setting = settings[0]
        self.assertEqual(setting['ExtentSize'], goal['ExtentSize'])
        self.assertEqual(setting['DataRedundancyGoal'], goal['DataRedundancyGoal'])
        self.assertLessEqual(setting['DataRedundancyMax'], goal['DataRedundancyMax'])
        self.assertGreaterEqual(setting['DataRedundancyMin'], goal['DataRedundancyMin'])
        self.assertEqual(setting['ExtentStripeLength'], goal['ExtentStripeLength'])
        self.assertLessEqual(setting['ExtentStripeLengthMax'], goal['ExtentStripeLengthMax'])
        self.assertGreaterEqual(setting['ExtentStripeLengthMin'], goal['ExtentStripeLengthMin'])
        self.assertEqual(setting['NoSinglePointOfFailure'], goal['NoSinglePointOfFailure'])
        self.assertEqual(setting['PackageRedundancyGoal'], goal['PackageRedundancyGoal'])
        self.assertLessEqual(setting['PackageRedundancyMax'], goal['PackageRedundancyMax'])
        self.assertGreaterEqual(setting['PackageRedundancyMin'], goal['PackageRedundancyMin'])

        (ret, outparams) = self.invoke_async_method(
                'DeleteStoragePool',
                self.service,
                int, None,
                Pool=vgname)
        self.assertEquals(ret, 0)
        self._delete_setting(goal.path)

    def test_create_setting_64k(self):
        """
            Test CreateOrModifyStoragePool with 64k ExtentSize.
        """
        goal = self._create_setting()
        goal['ExtentSize'] = pywbem.Uint64(64 * 1024)
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance,
                goal)
        self._delete_setting(goal.path)

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for SettingManager and Setting classes.

SettingManager
--------------

.. autoclass:: SettingManager
    :members:

Setting
-------

.. autoclass:: Setting
    :members:

StorageSetting
--------------

.. autoclass:: StorageSetting
    :members:

"""

import os
import ConfigParser
import lmi.providers.cmpi_logging as cmpi_logging

class SettingManager(object):
    """
        Class, which manages all persistent, transient and preconfigured
        LMI_*Setting instances.

        Note: LMI_*Setting instances, which represent actual configuration
        of some element are *not* managed by this class!

        It should be enough to have only one instance of this class.

        Settings must be removed using special methods of this class.

        Preconfigured settings are stored in /etc/openlmi/storage/setting/
        directory. Each LMI_*Setting class has its own file. Name of the
        file is the same as name of the class.
        Each file has ini structure. Each section represents one LMI_*Setting
        instance, with key=value pairs. Name of the ini section is the same
        as InstanceID of the setting.

        Persistent settings have the same structure, but they are stored in
        /var/lib/openlmi-storage/settings/ directory.
    """

    @cmpi_logging.trace_method
    def __init__(self, storage_configuration, timer_manager):
        """
        Create new SettingManager.

        :param storage_configuration: (``StorageConfiguration``) Current
                configuration.
        :param timer_manager: (``TimerManager``) Timer manager instance.
        """
        # hash classname -> settings
        #   settings = hash setting_id -> Setting
        self.classes = {}
        self.config = storage_configuration
        self.timer_manager = timer_manager
        # hash classname -> last generated unique ID (integer)
        self.ids = {}

    @cmpi_logging.trace_method
    def get_settings(self, classname):
        """
            Return dictionary of all instances of given LMI_*Setting class.
        """
        if not self.classes.has_key(classname):
            self.classes[classname] = {}
        return self.classes[classname]

    @cmpi_logging.trace_method
    def clean(self):
        """
            Remove all persistent and preconfigured settings, leaving
            only transient ones.
        """
        for cls in self.classes.values():
            for setting in cls.values():
                if (setting.type == Setting.TYPE_PERSISTENT
                        or setting.type == Setting.TYPE_PRECONFIGURED):
                    del(cls[setting.the_id])

    @cmpi_logging.trace_method
    def load(self):
        """
            Load all persistent and preconfigured settings from configuration
            files.
        """
        self.clean()

        # open all preconfigured config files
        self._load_directory(os.path.join(self.config.config_directory_provider(),
            self.config.SETTINGS_DIR), Setting.TYPE_PRECONFIGURED)
        self._load_directory(os.path.join(self.config.persistent_path(),
                    self.config.SETTINGS_DIR), Setting.TYPE_PERSISTENT)

    @cmpi_logging.trace_method
    def _load_directory(self, directory, setting_type):
        """ Load all ini files from given directory. """
        if not os.path.isdir(directory):
            return

        for classname in os.listdir(directory):
            ini = ConfigParser.SafeConfigParser()
            ini.optionxform = str  # don't convert to lowercase
            ini.read(directory + classname)
            for sid in ini.sections():
                setting = self.create_setting(classname, setting_type, sid)
                setting.load(ini)
                self._set_setting(classname, setting)

    @cmpi_logging.trace_method
    def _set_setting(self, classname, setting):
        """ Set given setting. """
        if self.classes.has_key(classname):
            stg = self.classes[classname]
            stg[setting.the_id] = setting
        else:
            self.classes[classname] = { setting.the_id : setting}
        setting.touch()

    @cmpi_logging.trace_method
    def set_setting(self, classname, setting):
        """
            Add or set setting. If the setting is (or was) persistent, it will
            be immediately stored to disk.
        """
        was_persistent = False
        settings = self.classes.get(classname, None)
        if settings:
            old_setting = settings.get(setting.the_id, None)
            if old_setting and old_setting.type == Setting.TYPE_PERSISTENT:
                was_persistent = True

        self._set_setting(classname, setting)
        if setting.type == Setting.TYPE_PERSISTENT or was_persistent:
            self._save_class(classname)

    @cmpi_logging.trace_method
    def delete_setting(self, classname, setting):
        """
            Remove a setting. If the setting was persistent, it will
            be immediately removed from disk.
        """
        settings = self.classes.get(classname, None)
        if settings:
            old_setting = settings.get(setting.the_id, None)
            if old_setting:
                del(settings[setting.the_id])
                if old_setting.type == Setting.TYPE_PERSISTENT:
                    self._save_class(classname)

    @cmpi_logging.trace_method
    def save(self):
        """
            Save all persistent settings to configuration files.
            Create the persistent directory if it does not exist.
        """
        for classname in self.classes.keys():
            self._save_class(classname)

    @cmpi_logging.trace_method
    def _save_class(self, classname):
        """ Save all settings of given class to persistent ini file."""
        ini = ConfigParser.SafeConfigParser()
        ini.optionxform = str  # don't convert to lowercase
        for setting in self.classes[classname].values():
            if setting.type != Setting.TYPE_PERSISTENT:
                continue
            setting.save(ini)

        finaldir = os.path.join(self.config.persistent_path(),
                self.config.SETTINGS_DIR)
        if not os.path.isdir(finaldir):
            os.makedirs(finaldir)
        with open(finaldir + classname, 'w') as configfile:
            ini.write(configfile)

    @cmpi_logging.trace_method
    def allocate_id(self, classname):
        """
            Return new unique InstanceID for given LMI_*Setting class.
        """
        if not self.ids.has_key(classname):
            self.ids[classname] = 1

        i = self.ids[classname]
        settings = self.get_settings(classname)
        while settings.has_key("LMI:" + classname + ":" + str(i)):
            i = i + 1

        self.ids[classname] = i + 1
        return "LMI:" + classname + ":" + str(i)

    @cmpi_logging.trace_method
    def create_setting(self, classname, setting_type=None, setting_id=None,
            class_to_create=None):
        """
        Create new Setting instance.

        :param classname: (``string``) Name of related CIM LMI_*Setting class.
        :param setting_type: (``Setting.TYPE_*`` constant) Type of the setting
                to create.
        :param setting_id: (``string`` constant) ID of the new setting.
        :param class_to_create: (``Class``) Subclass of Setting, which should
                be instantiated.
        """
        if class_to_create:
            return class_to_create(self, classname, setting_type, setting_id)
        else:
            return Setting(self, classname, setting_type, setting_id)

    @cmpi_logging.trace_method
    def expire_setting(self, classname, the_id):
        """
        Removes expired setting.

        :param classname: (``string``) Name of LMI_*Setting CIM class.
        :param the_id: (``string``) ID of the setting instance, which has
                expired.
        """
        settings = self.get_settings(classname)
        if settings:
            old_setting = settings.get(the_id, None)
            if old_setting:
                if old_setting.type == Setting.TYPE_TRANSIENT:
                    # Remove transient settings only.
                    del(settings[the_id])

class Setting(object):
    """
        This class represents generic LMI_*Setting properties.
        Every instance has name, type and properties (key-value pairs).
        The value must be string!
    """

    # setting with ChangeableType = Persistent
    TYPE_PERSISTENT = 1
    # setting with ChangeableType = Transient
    TYPE_TRANSIENT = 2
    # setting with ChangeableType = Fixed, preconfigured by system admin
    TYPE_PRECONFIGURED = 3
    # setting with ChangeableType = Fixed, current configuration of real
    # managed element, usually associated to it
    TYPE_CONFIGURATION = 4

    # Time of life of transient setting, in seconds.
    TRAINSIENT_SETTING_LIFETIME = 3600

    @cmpi_logging.trace_method
    def __init__(self, setting_manager, classname, setting_type=None, setting_id=None):
        self.type = setting_type
        self.the_id = setting_id
        self.properties = {}
        self.manager = setting_manager
        self.classname = classname

        self.timer = None
        self.touch()

    @cmpi_logging.trace_method
    def load(self, config):
        """
            Load setting with self.the_id from given ini file
            (ConfigParser instance).
        """
        self.properties = {}
        for (key, value) in config.items(self.the_id):
            if value == "":
                value = None
            self.properties[key] = value

    @cmpi_logging.trace_method
    def save(self, config):
        """
            Save setting with self.the_id to given ini file
            (ConfigParser instance).
        """
        config.add_section(self.the_id)
        for (key, value) in self.properties.items():
            if value is None:
                value = ""
            config.set(self.the_id, key, value)

    @cmpi_logging.trace_method
    def __getitem__(self, key):
        return self.properties[key]

    @cmpi_logging.trace_method
    def __setitem__(self, key, value):
        self.properties[key] = value

    @cmpi_logging.trace_method
    def __delitem__(self, key):
        del self.properties[key]

    @cmpi_logging.trace_method
    def __len__(self):
        return len(self.properties)

    @cmpi_logging.trace_method
    def has_key(self, key):
        """ Emulate dict.has_key(). """
        return self.properties.has_key(key)

    @cmpi_logging.trace_method
    def get(self, key, default=None):
        """ Emulate dict.get(). """
        return self.properties.get(key, default)

    @cmpi_logging.trace_method
    def items(self):
        """
            Return all (key, value) properties.
        """
        return self.properties.items()

    @cmpi_logging.trace_method
    def touch(self):
        """
        Reset expiration timer of transient setting.
        """
        if self.timer:
            self.timer.cancel()
        if self.type == self.TYPE_TRANSIENT:
            self.timer = self.manager.timer_manager.create_timer(
                    "Setting " + self.the_id)
            self.timer.set_callback(
                    self.manager.expire_setting,
                    self.classname,
                    self.the_id)
            self.timer.start(self.TRAINSIENT_SETTING_LIFETIME)


class StorageSetting(Setting):
    """
        Setting for LMI_StorageSetting subclasses. It has all the redundancy
        parameters.
    """
    def set_setting(self, redundancy):
        """
            Set setting according to given DeviceProvider.Redundancy.
        """
        self['DataRedundancyGoal'] = redundancy.data_redundancy
        self['DataRedundancyMax'] = redundancy.data_redundancy
        self['DataRedundancyMin'] = redundancy.data_redundancy
        self['ExtentStripeLength'] = redundancy.stripe_length
        self['ExtentStripeLengthMin'] = redundancy.stripe_length
        self['ExtentStripeLengthMax'] = redundancy.stripe_length
        self['NoSinglePointOfFailure'] = redundancy.no_single_point_of_failure
        self['PackageRedundancyGoal'] = redundancy.package_redundancy
        self['PackageRedundancyMax'] = redundancy.package_redundancy
        self['PackageRedundancyMin'] = redundancy.package_redundancy
        self['ParityLayout'] = redundancy.parity_layout



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LVBasedOn class.

LMI_LVBasedOn
-------------

.. autoclass:: LMI_LVBasedOn
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import blivet
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_LVBasedOn(BaseProvider):
    """
        Implementation of LMI_LVBasedOn class.
        This class is not subclass of BasedOnProvider, because we need
        to associate a LV with all PVs of the VG.
        BasedOnProvider would associate LV with its VG.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LVBasedOn, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'Dependent': None, 'Antecedent': None})

        for device in self.storage.lvs:
            provider = self.provider_manager.get_provider_for_device(device)
            if not provider:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Cannot find provider for device " + device.path)
            vg = device.vg
            for base in vg.parents:
                model['Dependent'] = provider.get_name_for_device(device)
                model['Antecedent'] = self.provider_manager.get_name_for_device(
                        base)
                if keys_only:
                    yield model
                else:
                    yield self.get_instance(env, model, device, base)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, device=None, base=None):
        """
            Provider implementation of GetInstance intrinsic method.
            It just checks if Dependent and Antecedent are related.
        """
        if not device:
            device = self.provider_manager.get_device_for_name(
                    model['Dependent'])
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Dependent device")

        if not base:
            base = self.provider_manager.get_device_for_name(
                    model['Antecedent'])
        if not base:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Antecedent device")

        if not isinstance(device,
                    blivet.devices.LVMLogicalVolumeDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Dependend device is not logical volume: " + device.path)
        if not (base in device.vg.parents):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Antecedent is not related to Dependent device")

        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_StorageExtent",
                "CIM_StorageExtent")



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase, short_tests_only
import unittest
import pywbem

MEGABYTE = 1024 * 1024

class TestCreateLV(StorageTestBase):
    """
        Test CreateOrModifyLV method.
    """

    VG_CLASS = "LMI_VGStoragePool"
    STYLE_GPT = 3
    PARTITION_CLASS = "LMI_GenericDiskPartition"


    def setUp(self):
        """ Find storage service. """
        super(TestCreateLV, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        self.part_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]

        vgname = self._create_vg(self.partition_names[:1])
        self.vg = self.wbemconnection.GetInstance(vgname)
        self.lvcaps_name = self.wbemconnection.AssociatorNames(vgname,
                AssocClass="LMI_LVElementCapabilities")[0]

    def tearDown(self):
        self._destroy_vg(self.vg.path)
        super(TestCreateLV, self).tearDown()

    def _create_vg(self, devicenames, name="tstName"):
        """
            Create a partition and Volume Group on it and return its
            CIMInstanceName.
        """
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyVG",
                self.service,
                int, "pool",
                InExtents=devicenames,
                ElementName=name)
        self.assertEqual(ret, 0)
        return outparams['pool']

    def _destroy_vg(self, vgname):
        """ Destroy VG and its partition. """
        self.invoke_async_method("DeleteVG", self.service, int, None,
                Pool=vgname)

    def test_create_no_pool(self):
        """ Test CreateOrModifyLV without InPool."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyLV",
                self.service,
                Size=pywbem.Uint64(40 * MEGABYTE))

    def test_create_no_size(self):
        """ Test CreateOrModifyLV without Size."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyLV",
                self.service,
                InPool=self.vg.path)

    def test_create_wrong_size(self):
        """ Test CreateOrModifyLV with wrong Size."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyLV",
                self.service,
                InPool=self.vg.path,
                Size=pywbem.Uint64(0))
# TODO: test this:
#        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
#                "CreateOrModifyLV",
#                self.service,
#                InPool=self.vg.path,
#                Size=pywbem.Uint64(self.vg['TotalManagedSpace'] * 10))

    def test_create_missing_goal(self):
        """ Test CreateOrModifyLV with missing Goal."""
        goal_name = pywbem.CIMInstanceName(
                classname="LMI_LVStorageSetting",
                keybindings={
                        "InstanceID": "LMI:LMI_LVStorageSetting:not-existing"
                })
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyLV",
                self.service,
                InPool=self.vg.path,
                Size=pywbem.Uint64(40 * MEGABYTE),
                Goal=goal_name)

    def _create_setting(self):
        """
            Create new LMI_LVStorageSetting with default values and return
            its CIMInstance.
        """
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "CreateLVStorageSetting",
                self.lvcaps_name)
        self.assertEqual(ret, 0)
        setting_name = outparams['setting']
        setting = self.wbemconnection.GetInstance(setting_name)
        return setting

    def test_create_wrong_goal(self):
        """ Test CreateOrModifyLV with wrong Goal."""
        setting = self._create_setting()
        setting['ExtentStripeLengthMin'] = pywbem.Uint16(100)
        self.wbemconnection.ModifyInstance(setting)

        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyLV",
                self.service,
                InPool=self.vg.path,
                Size=pywbem.Uint64(40 * MEGABYTE),
                Goal=setting.path)

        self.wbemconnection.DeleteInstance(setting.path)

    def test_create_no_goal(self):
        """ Test CreateOrModifyLV without any Goal."""
        self._test_create_no_goal(self.vg.path)

    def test_create_no_goal_raid(self):
        """ Test CreateOrModifyLV without any Goal on MDRAID."""
        raidname = self._create_mdraid(self.partition_names[2:4], 0)
        vgname = self._create_vg([raidname], "tstRaid")
        self._test_create_no_goal(vgname)
        self._destroy_vg(vgname)
        self._delete_mdraid(raidname)

    def _test_create_no_goal(self, vgname):
        """ Test CreateOrModifyLV without any Goal on a VG."""
        vg = self.wbemconnection.GetInstance(vgname)
        (retval, outparams) = self.invoke_async_method(
                "CreateOrModifyLV",
                self.service,
                int, "TheElement",
                InPool=vgname,
                Size=pywbem.Uint64(10 * vg['ExtentSize']))
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('TheElement'):
                lv = self.wbemconnection.GetInstance(outparams['TheElement'])
                outparams['Size'] = lv['BlockSize'] * lv['NumberOfBlocks']

        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 2)
        self.assertEqual(outparams['Size'], 10 * vg['ExtentSize'])

        lv_name = outparams['TheElement']
        lv = self.wbemconnection.GetInstance(lv_name)
        vg_setting = self.wbemconnection.Associators(vgname,
                AssocClass="LMI_VGElementSettingData")[0]
        lv_setting = self.wbemconnection.Associators(lv_name,
                AssocClass="LMI_LVElementSettingData")[0]

        self.assertEqual(
                lv['BlockSize'] * lv['NumberOfBlocks'],
                10 * vg['ExtentSize'])
        self.assertEqual(
                lv['NoSinglePointOfFailure'],
                lv_setting['NoSinglePointOfFailure'])
        self.assertEqual(
                lv['NoSinglePointOfFailure'],
                vg_setting['NoSinglePointOfFailure'])
        self.assertEqual(
                lv['DataRedundancy'],
                lv_setting['DataRedundancyGoal'])
        self.assertEqual(
                lv['DataRedundancy'],
                vg_setting['DataRedundancyGoal'])
        self.assertEqual(
                lv['PackageRedundancy'],
                lv_setting['PackageRedundancyGoal'])
        self.assertEqual(
                lv['PackageRedundancy'],
                vg_setting['PackageRedundancyGoal'])
        self.assertEqual(
                lv['ExtentStripeLength'],
                lv_setting['ExtentStripeLength'])
        self.assertEqual(
                lv['ExtentStripeLength'],
                vg_setting['ExtentStripeLength'])
        self.assertEqual(lv['Primordial'], False)

        # check vg is reduced
        new_vg = self.wbemconnection.GetInstance(vgname)
        self.assertEqual(
                new_vg['RemainingExtents'],
                vg['RemainingExtents'] - 10)
        self.assertEqual(
                new_vg['RemainingManagedSpace'],
                vg['RemainingManagedSpace'] - 10 * vg['ExtentSize'])

        self.invoke_async_method("DeleteLV", self.service, int, None,
                TheElement=lv_name)

    def test_create_goal_name(self):
        """ Test CreateOrModifyLV with a Goal and elementname."""
        goal = self._create_setting()
        (retval, outparams) = self.invoke_async_method(
                "CreateOrModifyLV",
                self.service,
                int, "TheElement",
                InPool=self.vg.path,
                Size=pywbem.Uint64(10 * self.vg['ExtentSize']),
                Goal=goal.path,
                ElementName="tstNAME")
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('TheElement'):
                lv = self.wbemconnection.GetInstance(outparams['TheElement'])
                outparams['Size'] = lv['BlockSize'] * lv['NumberOfBlocks']
        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 2)
        self.assertEqual(outparams['Size'], 10 * self.vg['ExtentSize'])

        lv_name = outparams['TheElement']
        lv = self.wbemconnection.GetInstance(lv_name)
        lv_setting = self.wbemconnection.Associators(lv_name,
                AssocClass="LMI_LVElementSettingData")[0]

        self.assertEqual(lv['ElementName'], "tstNAME")
        self.assertEqual(
                lv['BlockSize'] * lv['NumberOfBlocks'],
                10 * self.vg['ExtentSize'])
        self.assertEqual(
                lv['NoSinglePointOfFailure'],
                lv_setting['NoSinglePointOfFailure'])
        self.assertEqual(
                lv['NoSinglePointOfFailure'],
                goal['NoSinglePointOfFailure'])
        self.assertEqual(
                lv['DataRedundancy'],
                lv_setting['DataRedundancyGoal'])
        self.assertEqual(
                lv['DataRedundancy'],
                goal['DataRedundancyGoal'])
        self.assertEqual(
                lv['PackageRedundancy'],
                lv_setting['PackageRedundancyGoal'])
        self.assertEqual(
                lv['PackageRedundancy'],
                goal['PackageRedundancyGoal'])
        self.assertEqual(
                lv['ExtentStripeLength'],
                lv_setting['ExtentStripeLength'])
        self.assertEqual(
                lv['ExtentStripeLength'],
                goal['ExtentStripeLength'])

        # check vg is reduced
        new_vg = self.wbemconnection.GetInstance(self.vg.path)
        self.assertEqual(
                new_vg['RemainingExtents'],
                self.vg['RemainingExtents'] - 10)
        self.assertEqual(
                new_vg['RemainingManagedSpace'],
                self.vg['RemainingManagedSpace'] - 10 * self.vg['ExtentSize'])

        self.wbemconnection.DeleteInstance(goal.path)
        self.invoke_async_method("DeleteLV", self.service, int, None,
                TheElement=lv_name)

    @unittest.skipIf(short_tests_only(), reason="Skipping long tests.")
    def test_create_10(self):
        """ Test CreateOrModifyLV 10x."""
        self._test_create_10(self.vg.path)

    @unittest.skipIf(short_tests_only(), reason="Skipping long tests.")
    def test_create_10_raid(self):
        """ Test CreateOrModifyLV 10x on MD RAID."""
        raidname = self._create_mdraid(self.partition_names[2:4], 0)
        vgname = self._create_vg([raidname], "tstRaid")
        self._test_create_10(vgname)
        self._destroy_vg(vgname)
        self._delete_mdraid(raidname)

    def _test_create_10(self, vgname):
        """ Test CreateOrModifyLV 10x on given VG."""
        vg = self.wbemconnection.GetInstance(vgname)
        lvs = []
        for i in range(10):
            (retval, outparams) = self.invoke_async_method(
                    "CreateOrModifyLV",
                    self.service,
                    int, "TheElement",
                    InPool=vgname,
                    Size=pywbem.Uint64(2 * vg['ExtentSize']),
                    )
            if len(outparams) == 1:
                # there is no Size returned, Pegasus does not support it yet
                # TODO: remove when pegasus supports embedded objects of unknown
                # types, rhbz#920763
                if outparams.has_key('TheElement'):
                    lv = self.wbemconnection.GetInstance(outparams['TheElement'])
                    outparams['Size'] = lv['BlockSize'] * lv['NumberOfBlocks']
            self.assertEqual(retval, 0)
            self.assertEqual(len(outparams), 2)
            self.assertEqual(outparams['Size'], 2 * vg['ExtentSize'])

            lv_name = outparams['TheElement']
            lv = self.wbemconnection.GetInstance(lv_name)
            lv_setting = self.wbemconnection.Associators(lv_name,
                    AssocClass="LMI_LVElementSettingData")[0]
            lvs.append(lv)

            self.assertEqual(
                    lv['BlockSize'] * lv['NumberOfBlocks'],
                    2 * vg['ExtentSize'])
            self.assertEqual(
                    lv['NoSinglePointOfFailure'],
                    lv_setting['NoSinglePointOfFailure'])
            self.assertEqual(
                    lv['DataRedundancy'],
                    lv_setting['DataRedundancyGoal'])
            self.assertEqual(
                    lv['PackageRedundancy'],
                    lv_setting['PackageRedundancyGoal'])
            self.assertEqual(
                    lv['ExtentStripeLength'],
                    lv_setting['ExtentStripeLength'])

            # check vg is reduced
            new_vg = self.wbemconnection.GetInstance(vgname)
            self.assertEqual(
                    new_vg['RemainingExtents'],
                    vg['RemainingExtents'] - (i + 1) * 2)
            self.assertEqual(
                    new_vg['RemainingManagedSpace'],
                    vg['RemainingManagedSpace'] - (i + 1) * 2 * vg['ExtentSize'])

        for lv in lvs:
            self.invoke_async_method("DeleteLV", self.service, int, None,
                    TheElement=lv.path)

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for SettingProvider class.

SettingProvider
---------------

.. autoclass:: SettingProvider
    :members:

ElementSettingDataProvider
--------------------------

.. autoclass:: ElementSettingDataProvider
    :members:

SettingHelperProvider
---------------------

.. autoclass:: SettingHelperProvider
    :members:

"""

import pywbem
import ast
from lmi.storage.BaseProvider import BaseProvider
from lmi.storage.SettingManager import Setting
import lmi.providers.cmpi_logging as cmpi_logging
import lmi.providers

class SettingProvider(BaseProvider):
    """
        Base of all LMI_*Setting providers.
        Every setting class can have:
            - number of fixed preconfigured instances
            - number of configurable persistent instances
            - number of configurable in-memory (transient) instances
            - instances associated to managed elements
        This class provides all four instance types.

        The setting itself is represented by dictionary of key -> value.

        Preconfigured instances are stored in /etc/openlmi/storage/settings/<setting_classname>.ini
        Persistent instances are stored in /var/lib/openlmi-storage/settings/<setting_classname>.ini
    """
    @cmpi_logging.trace_method
    def __init__(self,
            setting_classname,
            supported_properties,
            validate_properties=None,
            ignore_defaults=None,
            *args, **kwargs):
        """
            setting_classname = name of CIM class, which we provide
            supported_properties = hash property_name -> constructor
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            validate_properties = hash property_name -> validator
                validator is a function which takes pywbem (Uint32, bool ...)
                value as parameter and returns True, if the value is correct for
                the property. Not all properties do need to have a validator.
            ignore_defaults = hash property_name -> default value.
                If this value of the property is set, the ModifyInstance
                won't complain, but it will silently ignore the value.
                This is useful when someone tries to set default value
                of a property and the provider does not implement it.
        """
        self.setting_classname = setting_classname
        supported_properties['Caption'] = str
        supported_properties['ConfigurationName'] = str
        supported_properties['Description'] = str
        supported_properties['ChangeableType'] = pywbem.Uint16
        supported_properties['ElementName'] = str

        self.supported_properties = supported_properties
        self.validate_properties = validate_properties
        self.ignore_defaults = ignore_defaults

        super(SettingProvider, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enumerate_configurations(self):
        """
            Enumerate all instances of LMI_*Setting, which are attached
            to managed elements, i.e. are not transient, persistent nor
            preconfigured.

            This method returns iterabe with Setting instances.

            Subclasses should override this method.
        """
        return []

    @cmpi_logging.trace_method
    def parse_setting_id(self, instance_id):
        """
            InstanceID should have format LMI:<classname>:<myid>.
            This method checks, that the format is OK and returns the myid.
            It returns None if the format is not OK.
            This method can be used in get_configuration_for_id.
        """
        return lmi.providers.parse_instance_id(
                instance_id, self.setting_classname)


    @cmpi_logging.trace_method
    def create_setting_id(self, myid):
        """
            InstanceID should have format LMI:<classname>:<ID>.
            This method returns string LMI:<classname>:<myid>
        """
        return "LMI:" + self.setting_classname + ":" + myid

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_configuration_for_id(self, instance_id):
        """
            Return Setting instance for given instance_id.
            Return None if no such Setting is found.

            Subclasses should override this method.
        """
        return None

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_associated_element_name(self, instance_id):
        """
            Return CIMInstanceName for ElementSettingData association.
            Return None if no such element exist.
            Subclasses should override this method.
        """
        return None

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
            Subclasses should not override this method, they should override
            enumerate_configurations only.
        """
        model.path.update({'InstanceID': None})

        # handle transient, persistent and preconfigured settings
        settings = self.setting_manager.get_settings(self.setting_classname)
        for setting in settings.values():
            model['InstanceID'] = setting.the_id
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, setting)
                # Make sure we will not return old values in next loop.
                for key in model.keys():
                    del model[key]

        # handle configurations
        for setting in self.enumerate_configurations():
            model['InstanceID'] = setting.the_id
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, setting)

    @cmpi_logging.trace_method
    def find_instance(self, instance_id):
        """
            Find an Setting instance with given InstanceID and return it.
            Return None if there is no such instance.
        """
        # find the setting in setting_manager
        settings = self.setting_manager.get_settings(self.setting_classname)
        if settings.has_key(instance_id):
            return settings[instance_id]

        # find the setting in configurations
        return self.get_configuration_for_id(instance_id)

    # pylint: disable-msg=W0221
    @cmpi_logging.trace_method
    def get_instance(self, env, model, setting=None):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        if not setting:
            setting = self.find_instance(model['InstanceID'])

        if not setting:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find setting.")

        # convert setting to model using supported_properties
        for (name, value) in setting.items():
            if value is not None:
                if self.supported_properties.has_key(name):
                    model[name] = self.supported_properties[name](value)

        types = self.Values.ChangeableType
        if setting.type == Setting.TYPE_CONFIGURATION:
            model['ChangeableType'] = types.Not_Changeable_Transient
        elif setting.type == Setting.TYPE_PERSISTENT:
            model['ChangeableType'] = types.Changeable_Persistent
        elif setting.type == Setting.TYPE_PRECONFIGURED:
            model['ChangeableType'] = types.Not_Changeable_Persistent
        elif setting.type == Setting.TYPE_TRANSIENT:
            model['ChangeableType'] = types.Changeable_Transient

        return model

    @staticmethod
    @cmpi_logging.trace_function
    def string_to_bool(value):
        """
            Convert a string to boolean value.
            '1', 'true' and 'True' are True, the rest is False.
        """
        if value == 1 or value == "true" or value == "True":
            return True
        elif value == 0 or value == "false" or value == "False":
            return False
        return bool(value)

    @staticmethod
    @cmpi_logging.trace_function
    def string_to_uint16_array(value):
        """
            Convert a string to array of integers.
            The string must be enclosed in [].
        """
        if not (value[0] == "[" and value[-1] == "]"):
            return None
        value = value[1:-1]
        values = value.split(",")
        return [pywbem.Uint16(i) for i in values]

    @staticmethod
    @cmpi_logging.trace_function
    def string_to_uint64_array(value):
        """
            Convert a string to array of integers.
            The string must be enclosed in [].
        """
        if not (value[0] == "[" and value[-1] == "]"):
            return None
        value = value[1:-1]
        values = value.split(",")
        return [pywbem.Uint64(i) for i in values]

    @staticmethod
    @cmpi_logging.trace_function
    def string_to_string_array(value):
        """
            Convert a string to array of strings.
            The string must be enclosed in [].
        """
        if not (value[0] == "[" and value[-1] == "]"):
            return None
        return ast.literal_eval(value)

    @cmpi_logging.trace_function
    def _check_changeable_type_modify(self, instance, setting):
        """
            Return True, if ModifInstance(instance) can modify ChangeableType
            property Setting instance.
            Return False, if the property should be skipped.
            Raise exception on error.
        """
        types = self.Values.ChangeableType
        if setting.type == Setting.TYPE_TRANSIENT:
            if instance['ChangeableType'] == types.Changeable_Persistent:
                # Can change only transient -> persistent
                # -> modify it now
                setting.type = Setting.TYPE_PERSISTENT
                return False
            if instance['ChangeableType'] == types.Changeable_Transient:
                # Ignore transient -> transient
                return False
        elif (setting.type == Setting.TYPE_PERSISTENT
                    and instance['ChangeableType']
                        == types.Changeable_Persistent):
            # Ignore persistent -> persistent
            return False
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                "Cannot modify ChangeableType property to new value.")

    @cmpi_logging.trace_function
    def _check_property_modify(self, instance, setting, property_name):
        """
            Return True, if ModifInstance(instance) can modify property of
            this name in given Setting instance.
            Return False, if the property should be skipped.
            Raise exception on error.
        """
        if property_name == 'InstanceID':
            # Ignore InstanceID property
            return False

        if self.ignore_defaults and self.ignore_defaults.has_key(property_name):
            # Ignore properties in self. ignore_defaults
            if self.ignore_defaults[property_name] != instance[property_name]:
                # But only if they have the default value
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Property is not supported: " + property_name)
            return False

        if not self.supported_properties.has_key(property_name):
            # We do not support the property
            if instance[property_name]:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Property is not supported: " + property_name)
            return False

        if property_name == 'ChangeableType':
            # ChangeableType has special treatment, only some changes are
            # allowed
            return self._check_changeable_type_modify(instance, setting)

        # Finally, allow the property to be set to new value.
        return True

    @cmpi_logging.trace_function
    def _do_modify_instance(self, instance, setting):
        """
            Modify instance of Setting with given CIMInstance.
        """
        for name in instance.iterkeys():
            if not self._check_property_modify(instance, setting, name):
                continue

            if instance[name] is not None:
                if (self.validate_properties
                        and self.validate_properties.has_key(name)):
                    # check validity of the property
                    validator = self.validate_properties[name]
                    if not validator(instance[name]):
                        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                "The value of property %s is invalid." % (name))
                setting[name] = str(instance[name])
            else:
                setting[name] = None
        return setting

    @cmpi_logging.trace_function
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        setting = self.find_instance(instance['InstanceID'])
        if not setting:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find setting.")

        if not modify_existing:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "CreateInstance is not supported.")

        if (setting.type == Setting.TYPE_CONFIGURATION
                or setting.type == Setting.TYPE_PRECONFIGURED):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Cannot modify not-changeable setting.")

        setting = self._do_modify_instance(instance, setting)

        self.setting_manager.set_setting(self.setting_classname, setting)
        return instance

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        setting = self.find_instance(instance_name['InstanceID'])
        if not setting:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find setting.")

        if (setting.type == Setting.TYPE_CONFIGURATION
                or setting.type == Setting.TYPE_PRECONFIGURED):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Cannot delete not-changeable setting.")

        self.setting_manager.delete_setting(self.setting_classname, setting)

    @cmpi_logging.trace_method
    def cim_method_clonesetting(self, env, object_name):
        """Implements LMI_DiskPartitionConfigurationSetting.CloneSetting()

        Create a copy of this instance. The resulting instance will have
        the same class and the same properties as the original instance
        except ChangeableType, which will be set to "Changeable -
        Transient" in the clone, and InstanceID.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CloneSetting()
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Clone -- (type REF (pywbem.CIMInstanceName(setting_classname='CIM_StorageSetting', ...))
            Created copy.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        setting = self.find_instance(object_name['InstanceID'])
        if not setting:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find setting.")

        instance_id = self.setting_manager.allocate_id(self.setting_classname)
        new_setting = self.setting_manager.create_setting(
                self.setting_classname,
                Setting.TYPE_TRANSIENT,
                instance_id)
        for (key, value) in setting.items():
            new_setting[key] = value
        self.setting_manager.set_setting(self.setting_classname, new_setting)

        out_params = []
        out_params += [pywbem.CIMParameter('Clone', type='reference',
                           value=pywbem.CIMInstanceName(
                                   classname=self.setting_classname,
                                   namespace=self.config.namespace,
                                   keybindings={'InstanceID' : instance_id}))]
        return (self.Values.CloneSetting.Success, out_params)


    class Values(object):
        class ChangeableType(object):
            Not_Changeable_Persistent = pywbem.Uint16(0)
            Changeable_Transient = pywbem.Uint16(1)
            Changeable_Persistent = pywbem.Uint16(2)
            Not_Changeable_Transient = pywbem.Uint16(3)

        class CloneSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Failed = pywbem.Uint32(4)


class ElementSettingDataProvider(BaseProvider):
    """
        Implementation of CIM_ElementSettingData.
        It uses functionality provided by SettingProvider.
    """
    @cmpi_logging.trace_method
    def __init__(self, setting_provider,
            managed_element_classname,
            setting_data_classname,
            *args, **kwargs):
        self.setting_provider = setting_provider
        self.managed_element_classname = managed_element_classname
        self.setting_data_classname = setting_data_classname
        super(ElementSettingDataProvider, self).__init__(*args, **kwargs)


    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        instance_id = model['SettingData']['InstanceID']
        element_name = self.setting_provider.get_associated_element_name(
                 instance_id)

        if not element_name:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the ManagedElement")

        if element_name != model['ManagedElement']:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The ManagedElement is not associated to given SettingData")

        model['IsCurrent'] = pywbem.Uint16(1)  # current
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'ManagedElement': None, 'SettingData': None})
        for setting in self.setting_provider.enumerate_configurations():
            instance_id = setting.the_id
            provider = self.setting_provider
            model['ManagedElement'] = provider.get_associated_element_name(
                    instance_id)
            model['SettingData'] = pywbem.CIMInstanceName(
                    classname=self.setting_data_classname,
                    namespace=self.config.namespace,
                    keybindings={'InstanceID' : instance_id})
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                               result_role, keys_only):
        # If you want to get references for free, implemented in terms
        # of enum_instances, just leave the code below unaltered.
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                self.managed_element_classname,
                self.setting_data_classname)


class SettingHelperProvider(SettingProvider):
    """
        Provider of LMI_*Setting class for managed element classes which
        implement SettingHelper.
    """

    @cmpi_logging.trace_method
    def __init__(self, setting_helper, *args, **kwargs):
        self.setting_helper = setting_helper
        properties = setting_helper.get_supported_setting_properties(self)
        validators = setting_helper.get_setting_validators(self)
        ignore = setting_helper.get_setting_ignore(self)
        super(SettingHelperProvider, self).__init__(
                supported_properties=properties,
                validate_properties=validators,
                ignore_defaults=ignore,
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enumerate_configurations(self):
        """
            Enumerate all instances of LMI_*Setting, which are attached
            to managed elements, i.e. are not transient, persistent nor
            preconfigured.
            It returns setting_helper.enumerate_settings.
        """
        return self.setting_helper.enumerate_settings(self)

    @cmpi_logging.trace_method
    def get_configuration_for_id(self, instance_id):
        """
            Return Setting instance for given instance_id.
            Return None if no such Setting is found.
        """
        return self.setting_helper.get_setting_for_id(self, instance_id)

    @cmpi_logging.trace_method
    def get_associated_element_name(self, instance_id):
        """
            Return CIMInstanceName for ElementSettingData association.
            Return None if no such element exist.
        """
        return self.setting_helper.get_associated_element_name(
                self, instance_id)



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
# -*- coding: utf-8 -*-
"""
Python Provider for LMI_MountedFileSystem

LMI_MountedFileSystem
---------------------

.. autoclass:: LMI_MountedFileSystem
    :members:

"""

import pywbem
import blivet
from lmi.storage.BaseProvider import BaseProvider
from lmi.storage.SettingHelper import SettingHelper
from lmi.storage.SettingManager import StorageSetting
from lmi.storage.SettingProvider import SettingProvider, Setting
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_MountedFileSystem(BaseProvider, SettingHelper):
    """Instrument the CIM class LMI_MountedFileSystem

    Class for representing mounted filesystems. Can be thought of as either
    an entry in /etc/mtab, or in /etc/fstab, according to its associated
    LMI_MountedFileSystemSetting.

    """
    MOUNT_OPTS = {
        'sync'          : ('SynchronousIO', 'True'),
        'dirsync'       : ('SynchronousDirectoryUpdates', 'True'),
        'atime'         : ('UpdateAccessTimes', 'True'),
        'strictatime'   : ('UpdateFullAccessTimes', 'True'),
        'relatime'      : ('UpdateRelativeAccessTimes', 'True'),
        'diratime'      : ('UpdateDirectoryAccessTimes', 'True'),
        'dev'           : ('InterpretDevices', 'True'),
        'mand'          : ('AllowMandatoryLock', 'True'),
        'exec'          : ('AllowExecution', 'True'),
        'suid'          : ('AllowSUID', 'True'),
        'rw'            : ('AllowWrite', 'True'),
        'silent'        : ('Silent', 'True'),
        'auto'          : ('Auto', 'True'),
        'user'          : ('AllowUserMount', 'True'),

        'async'         : ('SynchronousIO', 'False'),
        'noatime'       : ('UpdateAccessTimes', 'False'),
        'nostrictatime' : ('UpdateFullAccessTimes', 'False'),
        'norelatime'    : ('UpdateRelativeAccessTimes', 'False'),
        'nodiratime'    : ('UpdateDirectoryAccessTimes', 'False'),
        'nodev'         : ('InterpretDevices', 'False'),
        'nomand'        : ('AllowMandatoryLock', 'False'),
        'noexec'        : ('AllowExecution', 'False'),
        'nosuid'        : ('AllowSUID', 'False'),
        'ro'            : ('AllowWrite', 'False'),
        'loud'          : ('Silent', 'False'),
        'noauto'        : ('Auto', 'False'),
        'nouser'        : ('AllowUserMount', 'False')
    }

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        """
            Initialize the provider.
            Store reference to blivet.Blivet.
            Store reference to StorageConfiguration.
            Register at given ProviderManager.
        """
        super(LMI_MountedFileSystem, self).__init__(
            setting_classname='LMI_MountedFileSystemSetting',
            *args, **kwargs)
        self.classname = 'LMI_MountedFileSystem'

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """

        spec = model['FileSystemSpec']
        path = model['MountPointPath']
        device = self.storage.devicetree.getDeviceByPath(spec)
        not_mounted_ex = pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                         "%s is not mounted here: %s" % (spec, path))

        if isinstance(device, blivet.devices.NoDevice):
            if not self.storage.mountpoints.has_key(path):
                raise not_mounted_ex
            else:
                device = self.storage.mountpoints[path]
        else:
            if device is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "No such mounted device: " + spec)
            if path not in blivet.util.get_mount_paths(spec):
                raise not_mounted_ex

        model['InstanceID'] = self._create_instance_id(self._create_id(spec, path))
        model['FileSystemType'] = device.format.type

        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.
        """

        def _yield_instance(env, model, keys_only):
            if keys_only:
                return model
            else:
                try:
                    return self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND,
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model.
        model.path.update({'MountPointPath': None, 'FileSystemSpec': None})

        for device in self.storage.devices:
            if isinstance(device, blivet.devices.NoDevice):
                model['MountPointPath'] = device.format.mountpoint
                model['FileSystemSpec'] = device.path
                yield _yield_instance(env, model, keys_only)
            else:
                for path in blivet.util.get_mount_paths(device.path):
                    model['MountPointPath'] = path
                    model['FileSystemSpec'] = device.path
                    yield _yield_instance(env, model, keys_only)

    def _create_instance_id(self, mountid):
        return 'LMI:' + self.classname + ':' + mountid

    def _create_setting_instance_id(self, mountid):
        return 'LMI:' + self.classname + 'Setting:' + mountid

    def _create_id(self, spec, path):
        return spec + '|' + path

    def _parse_id(self, mountid):
        return mountid.split('|')

    def _get_options_from_name(self, opt):
        if opt in self.MOUNT_OPTS:
            return self.MOUNT_OPTS[opt]
        return (None, None)

    @cmpi_logging.trace_method
    def _get_setting_for_mount(self, device, path, setting_provider):
        """
            Return setting for given device.
        """
        _id = self._create_setting_instance_id(self._create_id(device.path, path))
        setting = self.setting_manager.create_setting(self.setting_classname,
                                                      Setting.TYPE_CONFIGURATION,
                                                      _id)
        setting['ElementName'] = _id
        other = []
        for opt in device.format.options.split(','):
            o, v = self._get_options_from_name(opt)
            if o:
                setting[o] = v
            else:
                other.append(opt)
        if other:
            setting['OtherOptions'] = str(other)
        return setting

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def enumerate_settings(self, setting_provider):
        """
            This method returns iterable with all instances of LMI_*Setting
            as Setting instances.
        """
        for device in self.storage.devices:
            if isinstance(device, blivet.devices.NoDevice):
                yield self._get_setting_for_mount(device, device.format.mountpoint, setting_provider)
            else:
                for path in blivet.util.get_mount_paths(device.path):
                    yield self._get_setting_for_mount(device, path, setting_provider)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_setting_for_id(self, setting_provider, instance_id):
        """
            Return Setting instance, which corresponds to LMI_*Setting with
            given InstanceID.
            Return None if there is no such instance.

            Subclasses must override this method.
        """
        mountid = setting_provider.parse_setting_id(instance_id)
        if not mountid:
            return None

        spec, path = self._parse_id(mountid)
        device = self.storage.devicetree.getDeviceByPath(spec)

        if isinstance(device, blivet.devices.NoDevice):
            if not self.storage.mountpoints.has_key(path):
                return None
            else:
                device = self.storage.mountpoints[path]
        else:
            if device is None:
                return None
            if path not in blivet.util.get_mount_paths(spec):
                return None

        return self._get_setting_for_mount(device, path, setting_provider)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_associated_element_name(self, setting_provider, instance_id):
        """
            Return CIMInstanceName of ManagedElement for ElementSettingData
            association for setting with given ID.
            Return None if no such ManagedElement exists.
        """
        mountid = setting_provider.parse_setting_id(instance_id)
        if not mountid:
            return None

        spec, path = self._parse_id(mountid)

        name = pywbem.CIMInstanceName(
            self.classname,
            namespace=self.config.namespace,
            keybindings={
                'FileSystemSpec': spec,
                'MountPointPath': path
                })

        return name

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_supported_setting_properties(self, setting_provider):
        """
            Return hash property_name -> constructor.
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            This hash will be passed to SettingProvider.__init__
        """
        return {
            'AllowExecution'              : SettingProvider.string_to_bool,
            'AllowMandatoryLock'          : SettingProvider.string_to_bool,
            'AllowSUID'                   : SettingProvider.string_to_bool,
            'AllowUserMount'              : SettingProvider.string_to_bool,
            'AllowWrite'                  : SettingProvider.string_to_bool,
            'Auto'                        : SettingProvider.string_to_bool,
            'Caption'                     : str,
            'ChangeableType'              : pywbem.Uint16,
            'ConfigurationName'           : str,
            'Description'                 : str,
            'Dump'                        : SettingProvider.string_to_bool,
            'ElementName'                 : str,
            'FileSystemCheckOrder'        : pywbem.Uint16,
            'Generation'                  : pywbem.Uint64,
            'InterpretDevices'            : SettingProvider.string_to_bool,
            'OtherOptions' : SettingProvider.string_to_string_array,
            'Silent'                      : SettingProvider.string_to_bool,
            'SynchronousDirectoryUpdates' : SettingProvider.string_to_bool,
            'SynchronousIO'               : SettingProvider.string_to_bool,
            'UpdateAccessTimes'           : SettingProvider.string_to_bool,
            'UpdateDirectoryAccessTimes'  : SettingProvider.string_to_bool,
            'UpdateFullAccessTimes'       : SettingProvider.string_to_bool,
            'UpdateRelativeAccessTimes'   : SettingProvider.string_to_bool,
        }



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem
import subprocess

class TestBlockStatistics(StorageTestBase):
    """
    Test that statistics provided by LMI_BlockStatisticsData and
    LMI_BlockStatisticsService are roughly accurate.
    """

    def _write_data(self, kbytes, device):
        """
        Write given nr. of kilobytes to given device.
        :param kbytes: ``int`` Nr. of kilobybytes to write.
        :param device: ``string`` Name of the device file.
        """
        subprocess.call(["dd", "if=/dev/zero", "of=" + device, "bs=1024",
                "count=%d" % (kbytes)])

    def _read_data(self, kbytes, device):
        """
        Read given nr. of kilobytes from given device.
        :param kbytes: ``int`` Nr. of kilobybytes to read.
        :param device: ``string`` Name of the device file.
        """
        subprocess.call(["dd", "if=" + device, "of=/dev/zero", "bs=1024",
                "count=%d" % (kbytes)])

    def test_data(self):
        """
        Test LMI_BlockStatisticsData.
        """
        READ_CHUNK = 2 * 1024 # 2 megabytes (the unit is kilobyte here)
        WRITE_CHUNK = 4 * 1024 # 4 megabytes

        # Load LMI_BlockStatisticsData of each partition.
        initial_stats = []
        for partition in self.partition_names:
            stats = self.wbemconnection.Associators(
                    partition,
                    ResultClass="LMI_BlockStorageStatisticalData")
            self.assertEquals(len(stats), 1)
            stat = stats[0]
            initial_stats.append(stat)

        # Generate some load
        i = 0
        for part in self.partitions:
            # read READ_CHUNK*<partition index> megabytes
            self._read_data(READ_CHUNK*(i+1), part)
            # write WRITE_CHUNK*<partition index> megabytes
            self._write_data(WRITE_CHUNK*(i+1), part)
            i += 1

        i = 0
        for partition in self.partition_names:
            stats = self.wbemconnection.Associators(
                    partition,
                    ResultClass="LMI_BlockStorageStatisticalData")
            self.assertEquals(len(stats), 1)
            stat = stats[0]
            initial_stat = initial_stats[i]

            # Just common sense tests.
            self.assertEquals(stat['InstanceID'], initial_stat['InstanceID'])
            self.assertEquals(stat['ElementName'], initial_stat['ElementName'])
            self.assertEquals(stat['ElementType'], initial_stat['ElementType'])
            self.assertEquals(stat['ElementType'], 9)  # Extent
            self.assertGreater(
                    stat['IOTimeCounter'], initial_stat['IOTimeCounter'])
            self.assertGreater(
                    stat['KBytesWritten'], initial_stat['KBytesWritten'])
            self.assertGreater(
                    stat['KBytesWritten'], initial_stat['KBytesWritten'])
            self.assertGreater(stat['ReadIOs'], initial_stat['ReadIOs'])
            self.assertGreater(stat['TotalIOs'], initial_stat['TotalIOs'])
            self.assertGreater(stat['WriteIOs'], initial_stat['WriteIOs'])
            self.assertGreater(
                    stat['KBytesTransferred'],
                    initial_stat['KBytesTransferred'])

            # Test that the counter have expected values
            # From some reason, the read counter includes also writes...
            # WRITE_CHUNK*<partition index> were written + READ_CHUNK*<partition index> were read
            self.assertAlmostEquals(
                    stat['KBytesRead'],
                    initial_stat['KBytesRead'] + (READ_CHUNK + WRITE_CHUNK)*(i+1),
                    delta=3 * 256)
            # WRITE_CHUNK*<partition index> megabytes were written
            self.assertAlmostEquals(
                    stat['KBytesWritten'],
                    initial_stat['KBytesWritten'] + WRITE_CHUNK*(i+1),
                    delta=256)

            # Check the counters match each other
            self.assertEquals(stat['TotalIOs'],
                    stat['ReadIOs'] + stat['WriteIOs'])
            # sum can be different +/-1 Kb because of rounding
            self.assertAlmostEquals(stat['KBytesTransferred'],
                    stat['KBytesWritten'] + stat['KBytesRead'], delta=1)

            i += 1

    def test_get_collection(self):
        """
        Test LMI_BlockStatisticsService.GetStatisticsCollection
        """
        service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_BlockStatisticsService")[0]

        # load the stats
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "GetStatisticsCollection",
                service)
        self.assertEquals(ret, 0)
        self.assertEquals(len(outparams), 1)
        # Join all statistics[] into one string
        stats = "\n".join(outparams['statistics'])

        self.assertGreater(len(stats), 0)

        # Load and parse the column headers
        manifest = self.wbemconnection.EnumerateInstances(
                "LMI_BlockStatisticsManifest")[0]
        columns = manifest['CSVSequence']

        # The first column must be InstanceID
        self.assertEquals(columns[0], "InstanceID")


        # Check all the lines have the same value as related
        # LMI_BlockStorageStatisticalData instance
        for line in stats.split("\n"):
            try:
                i = None
                if line == '':
                    continue
                # Parse the line
                values = line.split(";")
                self.assertEquals(len(columns), len(values))

                # Get the LMI_BlockStorageStatisticalData
                instanceid = values[0]
                path = pywbem.CIMInstanceName(
                        classname="LMI_BlockStorageStatisticalData",
                        namespace="root/cimv2",
                        keybindings={ 'InstanceID': instanceid})
                stats = self.wbemconnection.GetInstance(path)

                # Finally, check all columns except StatisticsTime
                for i in xrange(len(columns)):
                    if columns[i] == "StatisticTime":
                        continue
                    value = stats[columns[i]]
                    if isinstance(value, pywbem.Uint64):
                        # Compare counters with grain of salt, e.g. system disk
                        # can be busy. Use 4% as tolerance and add +1 for
                        # rounding errors
                        self.assertAlmostEquals(int(values[i]), value,
                                delta=value * 0.04 + 1)
                    else:
                        self.assertEquals(values[i], str(value))
            except Exception:
                # Print something useful and not generic 'xyz' != 'abc'
                if i is not None:
                    column_name = columns[i]
                else:
                    column_name = ""
                print "Error on line: '%s', column %d(%s)" % (line, i,
                        column_name)
                raise


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LocalFileSystemProvider.

LocalFileSystemProvider
-----------------------

.. autoclass:: LocalFileSystemProvider
    :members:

"""

import pywbem
from lmi.storage.FormatProvider import FormatProvider
from lmi.storage.SettingHelper import SettingHelper
from lmi.storage.SettingManager import Setting
from lmi.storage.SettingProvider import SettingProvider
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.util import storage
import os

class LocalFileSystemProvider(FormatProvider, SettingHelper):
    """
        Abstract provider for local filesystems.
        Each provider must have .device_type property, which represents
        blivet.formats.<DeviceFormat child>.type of format it
        represents.
    """


    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LocalFileSystemProvider, self).__init__(*args, **kwargs)

        vals = LocalFileSystemProvider.Values
        self.fs_settings = {
            'ext2' : {
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT2,
                'DataExtentsSharing':
                    vals.DataExtentsSharing.No_Sharing,
                'FilenameCaseAttributes':
                    vals.FilenameCaseAttributes.Case_sensitive,
                'ObjectTypes': [
                        vals.ObjectTypes.inodes,
                        vals.ObjectTypes.files,
                        vals.ObjectTypes.files_directories,
                ],
                'NumberOfObjectsMin': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'NumberOfObjectsMax': [
                        pywbem.Uint64(2 ** 32 - 1),  # nr. of inodes
                        pywbem.Uint64(10 ** 18),  # nr. of files
                        pywbem.Uint64(0),  # nr. of files in dir.
                ],
                'NumberOfObjects': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'ObjectSize':  [
                        pywbem.Uint64(256),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),  # file size in directory
                ],
                'ObjectSizeMin':  [
                        pywbem.Uint64(128),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),  # file size in directory
                ],
                'ObjectSizeMax':  [
                        pywbem.Uint64(4096),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),  # file size in directory
                ],
                'FilenameFormats': [
                    vals.FilenameFormats.Unix
                ],
                'FilenameLengthMax':  [
                        pywbem.Uint64(255),
                ],
                'PersistenceTypes': [
                        vals.PersistenceTypes.Persistent]
            },
            'ext3' : {
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT3,
                'DataExtentsSharing':
                    vals.DataExtentsSharing.No_Sharing,
                'FilenameCaseAttributes':
                    vals.FilenameCaseAttributes.Case_sensitive,
                'ObjectTypes': [
                        vals.ObjectTypes.inodes,
                        vals.ObjectTypes.files,
                        vals.ObjectTypes.files_directories,
                ],
                'NumberOfObjectsMin': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'NumberOfObjectsMax': [
                        pywbem.Uint64(2 ** 32 - 1),  # nr.of inodes
                        pywbem.Uint64(2 ** 32 - 1),  # nr. of files
                        pywbem.Uint64(0),
                ],
                'NumberOfObjects': [
                        pywbem.Uint64(0),  # nr.of inodes
                        pywbem.Uint64(0),  # nr. of files
                        pywbem.Uint64(0),
                ],
                'ObjectSize':  [
                        pywbem.Uint64(256),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMin':  [
                        pywbem.Uint64(128),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMax':  [
                        pywbem.Uint64(4096),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'FilenameFormats': [
                    vals.FilenameFormats.Unix
                ],
                'FilenameLengthMax':  [
                        pywbem.Uint64(255),
                ],
                'PersistenceTypes': [
                        vals.PersistenceTypes.Persistent]
            },
            'ext4' : {
                'ActualFileSystemType':
                    vals.ActualFileSystemType.EXT4,
                'DataExtentsSharing':
                    vals.DataExtentsSharing.No_Sharing,
                'FilenameCaseAttributes':
                    vals.FilenameCaseAttributes.Case_sensitive,
                'ObjectTypes': [
                        vals.ObjectTypes.inodes,
                        vals.ObjectTypes.files,
                        vals.ObjectTypes.files_directories,
                ],
                'NumberOfObjectsMin': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'NumberOfObjectsMax': [
                        pywbem.Uint64(2 ** 32 - 1),  # nr.of inodes
                        pywbem.Uint64(2 ** 32 - 1),  # nr. of files
                        pywbem.Uint64(0),
                ],
                'NumberOfObjects': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'ObjectSize':  [
                        pywbem.Uint64(256),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMin':  [
                        pywbem.Uint64(128),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMax':  [
                        pywbem.Uint64(4096),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'FilenameFormats': [
                    vals.FilenameFormats.Unix
                ],
                'FilenameLengthMax':  [
                        pywbem.Uint64(255),
                ],
                'PersistenceTypes': [
                        vals.PersistenceTypes.Persistent]
            },
            'btrfs' : {
                'ActualFileSystemType':
                    vals.ActualFileSystemType.BTRFS,
                'DataExtentsSharing':
                    vals.DataExtentsSharing.No_Sharing,
                'FilenameCaseAttributes':
                    vals.FilenameCaseAttributes.Case_sensitive,
                'ObjectTypes': [
                        vals.ObjectTypes.inodes,
                        vals.ObjectTypes.files,
                        vals.ObjectTypes.files_directories,
                ],
                'NumberOfObjectsMin': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'NumberOfObjectsMax': [
                        pywbem.Uint64(2 ** 64 - 1),  # nr.of inodes
                        pywbem.Uint64(2 ** 64 - 1),  # nr. of files
                        pywbem.Uint64(0),
                ],
                'NumberOfObjects': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'ObjectSize':  [
                        pywbem.Uint64(0),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMin':  [
                        pywbem.Uint64(128),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMax':  [
                        pywbem.Uint64(4096),  # inode size
                        pywbem.Uint64(8 * (2 ** 60)),  # file size 8 EiB
                        pywbem.Uint64(0),
                ],
                'FilenameFormats': [
                    vals.FilenameFormats.Unix
                ],
                'FilenameLengthMax':  [
                        pywbem.Uint64(255),
                ],
                'PersistenceTypes': [
                        vals.PersistenceTypes.Persistent]
            },
            'xfs' : {
                'ActualFileSystemType':
                    vals.ActualFileSystemType.XFS,
                'DataExtentsSharing':
                    vals.DataExtentsSharing.No_Sharing,
                'FilenameCaseAttributes':
                    vals.FilenameCaseAttributes.Case_sensitive,
                'ObjectTypes': [
                        vals.ObjectTypes.inodes,
                        vals.ObjectTypes.files,
                        vals.ObjectTypes.files_directories,
                ],
                'NumberOfObjectsMin': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'NumberOfObjectsMax': [
                        pywbem.Uint64(2 ** 64 - 1),  # nr.of inodes
                        pywbem.Uint64(2 ** 64 - 1),  # nr of files
                        pywbem.Uint64(0),
                ],
                'NumberOfObjects': [
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                        pywbem.Uint64(0),
                ],
                'ObjectSize':  [
                        pywbem.Uint64(0),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMin':  [
                        pywbem.Uint64(128),  # inode size
                        pywbem.Uint64(0),  # file size
                        pywbem.Uint64(0),
                ],
                'ObjectSizeMax':  [
                        pywbem.Uint64(4096),  # inode size
                        pywbem.Uint64(8 * (2 ** 60) - 1),  # file size 8 EiB-1
                        pywbem.Uint64(0),
                ],
                'FilenameFormats': [
                    vals.FilenameFormats.Unix
                ],
                'FilenameLengthMax':  [
                        pywbem.Uint64(255),
                ],
                'PersistenceTypes': [
                        vals.PersistenceTypes.Persistent]
            },
        }


    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, fmt=None):
        """
            Get instance.
            Subclasses should override this method, the default implementation
            just check if the instance exists.
        """
        if not fmt:
            (device, fmt) = self.get_format_for_name(model)
        else:
            device = self.storage.devicetree.getDeviceByPath(fmt.device)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the format.")

        model['FileSystemType'] = fmt.name
        model['CaseSensitive'] = True
        model['CasePreserved'] = True
        model['PersistenceType'] = self.Values.PersistenceType.Persistent
        model['ElementName'] = fmt.device
        if fmt.resizable is None:
            model['IsFixedSize'] = self.Values.IsFixedSize.Not_Specified
        elif fmt.resizable:
            model['IsFixedSize'] = self.Values.IsFixedSize.Not_Fixed_Size
        else:
            model['IsFixedSize'] = self.Values.IsFixedSize.Fixed_Size
        uuid = self.get_uuid(device, fmt)
        if uuid:
            model['UUID'] = uuid
        if fmt.label:
            model['ElementName'] = fmt.label

        if fmt.mountpoint:
            model['Root'] = fmt.mountpoint
            # TODO: this should be provided by Blivet, see bug #915201
            stat = os.statvfs(fmt.mountpoint)
            model['BlockSize'] = pywbem.Uint64(stat.f_bsize)
            model['FileSystemSize'] = pywbem.Uint64(
                    stat.f_blocks * stat.f_frsize)
            model['AvailableSpace'] = pywbem.Uint64(
                    stat.f_bavail * stat.f_bsize)
            model['ReadOnly'] = (stat.f_flag & os.ST_RDONLY) > 0
            model['MaxFileNameLength'] = pywbem.Uint32(stat.f_namemax)

        return model

    @cmpi_logging.trace_method
    def _get_setting_for_format(self, setting_provider, fmt):
        """
            Return Setting for given format.
        """
        # Table of filesystems with Setting
        values = self.fs_settings.get(fmt.type, None)
        if values:
            # yes, we should create Setting for this FS
            setting = self.setting_manager.create_setting(
                    setting_provider.setting_classname,
                    Setting.TYPE_CONFIGURATION,
                    setting_provider.create_setting_id(fmt.device))

            if fmt.mountpoint:
                # Fill in current filesystem setting.
                # Copy the values dictionary, we are modifying it.
                values = values.copy()
                # TODO: this should be provided by Blivet, see bug #915201
                stat = os.statvfs(fmt.mountpoint)
                # [nr. of inodes, nr. of files, nr. of files in a directory]
                values['NumberOfObjects'] = [stat.f_files, stat.f_files, 0]
                values['FilenameLengthMax'] = [stat.f_namemax]

            for (key, value) in values.iteritems():
                setting[key] = str(value)

            return setting
        return None

    @cmpi_logging.trace_method
    def enumerate_settings(self, setting_provider):
        """
            This method returns iterable with all instances of LMI_*Setting
            as Setting instances.
        """
        for device in self.storage.devices:
            if self.provides_format(device, device.format):
                setting = self._get_setting_for_format(
                        setting_provider, device.format)
                if setting:
                    yield setting


    @cmpi_logging.trace_method
    def get_setting_for_id(self, setting_provider, instance_id):
        """
            Return Setting instance, which corresponds to LMI_*Setting with
            given InstanceID.
            Return None if there is no such instance.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        if not device.format:
            return None
        return self._get_setting_for_format(setting_provider, device.format)

    @cmpi_logging.trace_method
    def get_associated_element_name(self, setting_provider, instance_id):
        """
            Return CIMInstanceName of ManagedElement for ElementSettingData
            association for setting with given ID.
            Return None if no such ManagedElement exists.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        fmt = device.format
        provider = self.provider_manager.get_provider_for_format(device, fmt)
        return provider.get_name_for_format(device, fmt)

    @cmpi_logging.trace_method
    def get_supported_setting_properties(self, setting_provider):
        """
            Return hash property_name -> constructor.
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            This hash will be passed to SettingProvider.__init__
        """
        return {
                'ActualFileSystemType': pywbem.Uint16,
                'DataExtentsSharing': pywbem.Uint16,
                'FilenameCaseAttributes': pywbem.Uint16,
                'ObjectTypes' : SettingProvider.string_to_uint16_array,
                'NumberOfObjectsMin' : SettingProvider.string_to_uint64_array,
                'NumberOfObjectsMax' : SettingProvider.string_to_uint64_array,
                'NumberOfObjects' : SettingProvider.string_to_uint64_array,
                'ObjectSize' : SettingProvider.string_to_uint64_array,
                'ObjectSizeMin' : SettingProvider.string_to_uint64_array,
                'ObjectSizeMax' : SettingProvider.string_to_uint64_array,
                'PersistenceTypes' : SettingProvider.string_to_uint16_array,
                'FilenameFormats' : SettingProvider.string_to_uint16_array,
                'FilenameLengthMax' : SettingProvider.string_to_uint16_array,
        }

    @cmpi_logging.trace_method
    def get_setting_ignore(self, setting_provider):
        return {
                'CopyTagret': 0,
        }

    class Values(FormatProvider.Values):
        class PersistenceType(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Persistent = pywbem.Uint16(2)
            Temporary = pywbem.Uint16(3)
            External = pywbem.Uint16(4)

        class ActualFileSystemType(object):
            Unknown = pywbem.Uint16(0)
            UFS = pywbem.Uint16(2)
            HFS = pywbem.Uint16(3)
            FAT = pywbem.Uint16(4)
            FAT16 = pywbem.Uint16(5)
            FAT32 = pywbem.Uint16(6)
            NTFS4 = pywbem.Uint16(7)
            NTFS5 = pywbem.Uint16(8)
            XFS = pywbem.Uint16(9)
            AFS = pywbem.Uint16(10)
            EXT2 = pywbem.Uint16(11)
            EXT3 = pywbem.Uint16(12)
            REISERFS = pywbem.Uint16(13)
            # DMTF_Reserved = ..
            EXT4 = pywbem.Uint16(0x8001)
            BTRFS = pywbem.Uint16(0x8002)
            JFS = pywbem.Uint16(0x8003)
            TMPFS = pywbem.Uint16(0x8004)
            VFAT = pywbem.Uint16(0x8005)

        class FilenameCaseAttributes(object):
            Unknown = pywbem.Uint16(0)
            Case_sensitive = pywbem.Uint16(1)
            Case_forced_to_upper_case = pywbem.Uint16(2)
            Case_forced_to_lower_case = pywbem.Uint16(3)
            Case_indifferent_but_lost = pywbem.Uint16(4)
            Case_indifferent_but_preserved = pywbem.Uint16(5)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..

        class ObjectTypes(object):
            inodes = pywbem.Uint16(2)
            files = pywbem.Uint16(3)
            directories = pywbem.Uint16(4)
            links = pywbem.Uint16(5)
            devices = pywbem.Uint16(6)
            files_directories = pywbem.Uint16(7)
            Blocks = pywbem.Uint16(8)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..

        class PersistenceTypes(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Persistent = pywbem.Uint16(2)
            Temporary = pywbem.Uint16(3)
            External = pywbem.Uint16(4)
            # DMTF_Reserved = 5..

        class DataExtentsSharing(object):
            Unknown = pywbem.Uint16(0)
            No_Sharing = pywbem.Uint16(1)
            Sharing_Possible_Optional = pywbem.Uint16(2)
            Sharing_Enforced = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..

        class FilenameFormats(object):
            DOS8_3 = pywbem.Uint16(1)
            Unix = pywbem.Uint16(2)
            VMS = pywbem.Uint16(3)
            Windows_LongNames = pywbem.Uint16(4)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..

        class IsFixedSize(object):
            Not_Specified = pywbem.Uint16(0)
            Fixed_Size = pywbem.Uint16(1)
            Not_Fixed_Size = pywbem.Uint16(2)
            _reverse_map = {0: 'Not Specified', 1: 'Fixed Size', 2: 'Not Fixed Size'}
